/* 
* Rootfly:  A program for analyzing and measuring roots. 
* Copyright (c) 2007 Clemson University.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

// RootflyDlg.cpp : implementation file
//
#include "stdafx.h"
#include "Rootfly.h"
#include "RootflyDlg.h" 
#include "DebugDlg.h" 
#include "Image.h"		  
#include <math.h>
#include <vector>  
#include <iterator>
#include <string> 
#include <sstream> 
#include <direct.h>     
#include "OptionsDlg.h"            // file format dialog
#include "MMConversionDlg.h"          // calibration dialog
#include "ManageConditionCodesDlg.h"  // dialog for editing condition codes
#include "CopySession.h"           // copy session dialog
#include "ColorDlg.h"              // color dialog
#include "FolderDlg.h"             // file browse dialog
#include "ProgressData.h"
#include "ProgressDlg.h"
#include "AutoDetectDlg.h"
#include "RootDetect.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define NUM_THUMBNAILS 5

using namespace::std;

DebugLogFile g_debug_logfile;

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About
/**@class CAboutDlg
@ brief Bring up the version and copyright information about Rootfly 
*/
class CAboutDlg : public CDialog
{
public:
  CAboutDlg();
  
  // Dialog Data
  //{{AFX_DATA(CAboutDlg)
  enum { IDD = IDD_ABOUTBOX };
  //}}AFX_DATA
  
  // ClassWizard generated virtual function overrides
  //{{AFX_VIRTUAL(CAboutDlg)
protected:
  virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
  //}}AFX_VIRTUAL
  
  // Implementation
protected:
  //{{AFX_MSG(CAboutDlg)
  //}}AFX_MSG
  DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
  //{{AFX_DATA_INIT(CAboutDlg)
  //}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CAboutDlg)
  //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
//{{AFX_MSG_MAP(CAboutDlg)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRootflyDlg dialog

CRootflyDlg::CRootflyDlg(CWnd* pParent /*=NULL*/)
: CDialog(CRootflyDlg::IDD, pParent)
{
  m_controller.C_ReadUserPreferences();
  
  m_rfy_data = m_controller.GetRootflyData();
  m_set_data = m_rfy_data->GetSettings();
  m_exp_data = m_rfy_data->GetExperiment();
  m_app_data = m_rfy_data->GetApplicationData();
  m_graphics.Initialize( m_converter, *m_set_data, *m_rfy_data );
  
  //{{AFX_DATA_INIT(CRootflyDlg)
  
  //}}AFX_DATA_INIT
  // Note that LoadIcon does not require a subsequent DestroyIcon in Win32
  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
  
}

CRootflyDlg::~CRootflyDlg()
{
}

void CRootflyDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CRootflyDlg)	
	DDX_Control(pDX, IDC_STATIC_status, m_status_text);
  DDX_Control(pDX, IDC_BUTTON_Move_All_Roots, m_button_move_all_roots);
  DDX_Control(pDX, IDC_CHECK_Zoom_Out, m_check_zoom_out);
  DDX_Control(pDX, IDC_COMBO_COLOR, m_combo_color);
  DDX_Control(pDX, IDC_STATIC_MM, m_mm_text);
  DDX_Control(pDX, IDC_SESSION_DATE_5, m_session_date5);
  DDX_Control(pDX, IDC_SESSION_DATE_4, m_session_date4);
  DDX_Control(pDX, IDC_SESSION_DATE_3, m_session_date3);
  DDX_Control(pDX, IDC_SESSION_DATE_2, m_session_date2);
  DDX_Control(pDX, IDC_SESSION_DATE_1, m_session_date1);
  DDX_Control(pDX, IDC_STATIC_Root_ID, m_root_id_text); 
  DDX_Control(pDX, IDC_BUTTON_Delete_Root, m_button_delete_root);
  DDX_Control(pDX, IDC_BUTTON_Copy_Session, m_button_copy);
  DDX_Control(pDX, IDC_BUTTON_Extend_Endpoints, m_button_extend_endpoints);
  DDX_Control(pDX, IDC_BUTTON_New_Root, m_button_new_root);
  DDX_Control(pDX, IDC_STATIC_Color, m_color_text);
  DDX_Control(pDX, IDC_STATIC_Length, m_length_text);
  DDX_Control(pDX, IDC_STATIC_Session, m_session_text);
  DDX_Control(pDX, IDC_STATIC_Custom, m_custom_text);
  DDX_Control(pDX, IDC_STATIC_Diameter, m_diameter_text);
  DDX_Control(pDX, IDC_STATIC_State, m_state_text);
  DDX_Control(pDX, IDC_STATIC_Tube, m_tube_text);
  DDX_Control(pDX, IDC_STATIC_Window, m_window_text);
  DDX_Control(pDX, IDC_STATIC_Properties_Section, m_properties_section);
  DDX_Control(pDX, IDC_STATIC_Session_Section, m_session_section);
  DDX_Control(pDX, IDC_STATIC_Images_Section, m_images_section);
  DDX_Control(pDX, IDC_STATIC_Window_Section, m_window_section); 
  DDX_Control(pDX, IDC_CHECK_Show_Root_IDs, m_checkbox_show_root_ids);
  DDX_Control(pDX, IDC_CHECK_Show_Diameters, m_checkbox_show_diameters);
  DDX_Control(pDX, IDC_CHECK_Show_Dead_Roots, m_checkbox_show_dead_roots);
  DDX_Control(pDX, IDC_SCROLLBAR3, m_scrollbar_horizontal);
  DDX_Control(pDX, IDC_SCROLLBAR2, m_scrollbar_vertical);
  DDX_Control(pDX, IDC_WIND_PIC_1, m_Window_Pic_1);
  DDX_Control(pDX, IDC_WIND_PIC_2, m_Window_Pic_2);
  DDX_Control(pDX, IDC_WIND_PIC_3, m_Window_Pic_3);
  DDX_Control(pDX, IDC_WIND_PIC_4, m_Window_Pic_4);
  DDX_Control(pDX, IDC_WIND_PIC_5, m_Window_Pic_5);
  DDX_Control(pDX, IDC_SESS_PIC_1, m_Session_Pic_1);
  DDX_Control(pDX, IDC_SESS_PIC_2, m_Session_Pic_2);
  DDX_Control(pDX, IDC_SESS_PIC_3, m_Session_Pic_3);
  DDX_Control(pDX, IDC_SESS_PIC_4, m_Session_Pic_4);
  DDX_Control(pDX, IDC_SESS_PIC_5, m_Session_Pic_5);
  DDX_Control(pDX, IDC_EDIT_TUBE, m_edit_tube);
  DDX_Control(pDX, IDC_EDIT_WINDOW, m_edit_window);
  DDX_Control(pDX, IDC_SPIN_TUBE, m_spin_tube);
  DDX_Control(pDX, IDC_SPIN_WINDOW, m_spin_window);
  DDX_Control(pDX, IDC_SPIN_SESSION, m_spin_session);
  DDX_Control(pDX, IDC_EDIT_ROOTID, m_edit_rootid);	
  DDX_Control(pDX, IDC_EDIT_DIAMETER, m_edit_diameter);
  DDX_Control(pDX, IDC_EDIT_LENGTH, m_edit_length);
  DDX_Control(pDX, IDC_COMBO_STATE, m_combo_state);
  DDX_Control(pDX, IDC_COMBO_CUSTOM, m_combo_custom);
  DDX_Control(pDX, IDC_EDIT_SESSION, m_edit_session); 
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CRootflyDlg, CDialog)
//{{AFX_MSG_MAP(CRootflyDlg)
ON_WM_SYSCOMMAND()
ON_WM_PAINT()
ON_WM_QUERYDRAGICON()
ON_BN_CLICKED(IDC_BUTTON_New_Root, OnBUTTONNewRoot)
ON_BN_CLICKED(IDC_BUTTON_Extend_Endpoints, OnBUTTONExtendEndpoints)
ON_BN_CLICKED(IDC_BUTTON_Delete_Root, OnBUTTONDeleteRoot)
ON_BN_CLICKED(IDC_BUTTON_Copy_Session, OnBUTTONCopySession)
ON_COMMAND(ID_FILE_EXIT, OnFileExit)
ON_COMMAND(ID_FILE_NEW, OnFileNew)
ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
ON_COMMAND(ID_FILE_SAVEAS, OnFileSaveAs)
ON_COMMAND(ID_FILE_SAVE, OnFileSave)
ON_COMMAND(ID_EDIT_OPTIONS, OnEditOptions)
ON_COMMAND(ID_HELP_ABOUT, OnHelpAbout)
ON_WM_LBUTTONDOWN()
ON_WM_LBUTTONUP()
ON_WM_LBUTTONDBLCLK()
ON_WM_RBUTTONDOWN()
ON_WM_MOUSEMOVE()
ON_WM_SIZE()
ON_COMMAND(ID_HELP_CONTENTS, OnHelpContents)
ON_EN_KILLFOCUS(IDC_EDIT_DIAMETER, OnKillfocusEditDiameter)
ON_EN_KILLFOCUS(IDC_EDIT_SESSION, OnKillfocusEditSession)
ON_EN_KILLFOCUS(IDC_EDIT_WINDOW, OnKillfocusEditWindow)
ON_EN_KILLFOCUS(IDC_EDIT_TUBE, OnKillfocusEditTube)
ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_TUBE, OnDeltaposSpinTube)
ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_SESSION, OnDeltaposSpinSession)
ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_WINDOW, OnDeltaposSpinWindow)	
ON_BN_CLICKED(IDC_CHECK_Show_Root_IDs, OnCHECKShowRootIDs)
ON_BN_CLICKED(IDC_CHECK_Show_Dead_Roots, OnCHECKShowDeadRoots)
ON_BN_CLICKED(IDC_CHECK_Show_Diameters, OnCHECKShowDiameters)
ON_WM_VSCROLL()
ON_WM_HSCROLL()
ON_COMMAND(ID_FILE_EXPORT, OnFileExport)
ON_WM_CLOSE()
ON_COMMAND(ID_EDIT_CONVERSION, OnConversion)
ON_CBN_SELCHANGE(IDC_COMBO_COLOR, OnSelchangeComboColor)
ON_CBN_SELCHANGE(IDC_COMBO_CUSTOM, OnSelchangeComboCustom)
ON_CBN_SELCHANGE(IDC_COMBO_STATE, OnSelchangeComboState)
ON_BN_CLICKED(IDC_BUTTON_Move_All_Roots, OnButtonMoveAllRoots)
ON_BN_CLICKED(IDC_CHECK_Zoom_Out, OnCHECKZoomOut)
ON_CBN_DROPDOWN(IDC_COMBO_CUSTOM, OnDropdownComboCustom)
ON_CBN_KILLFOCUS(IDC_COMBO_CUSTOM, OnKillfocusComboCustom)
ON_COMMAND(ID_EDIT_CONDITION, OnCondition)
ON_COMMAND(ID_EDIT_CHANGE_IMG_DIR, OnEditChangeImgDir)
ON_COMMAND(ID_EDIT_COLORS, OnEditColors)
ON_COMMAND(ID_EDIT_SESSION_DATES, OnEditSessionDates)
ON_COMMAND(ID_MENUCONTEXT_DELETEPOINT, OnDeletePoint)
ON_COMMAND(ID_MENUCONTEXT_ADDPOINT, OnAddPoint)
ON_WM_MOUSEWHEEL()
ON_WM_HELPINFO()
ON_COMMAND(ID_DEBUG_RESETMODE, OnDebugResetmode)
ON_COMMAND(ID_DEBUG_DISPLAYROOTDATA, OnDebugDisplayrootdata)
ON_COMMAND(ID_DEBUG_REPAIRDIAMETER, OnDebugRepairdiameter)
ON_COMMAND(ID_DEBUG_SHOWDEBUGLOGFILELOCATION, OnDebugShowdebuglogfilelocation)
ON_COMMAND(ID_NAVIGATE_PROGRESSWINDOW, OnNavigateProgresswindow)
ON_COMMAND(ID_EDIT_ZOOM_OUT, OnEditZoomOut)
	ON_UPDATE_COMMAND_UI(ID_VIEW_RIGHTPANE, OnUpdateViewRightpane)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
	ON_COMMAND(ID_EDIT_AUTODETECT, OnEditAutodetect)
	ON_BN_CLICKED(IDC_BUTTON_Auto_Detect, OnBUTTONAutoDetect)
	ON_UPDATE_COMMAND_UI(ID_VIEW_LEFTPANE, OnUpdateViewLeftpane)
ON_WM_MOUSEWHEEL()
ON_EN_KILLFOCUS(IDC_EDIT_DIAMETER, OnKillfocusEditDiameter)
ON_CBN_EDITCHANGE(IDC_COMBO_STATE, OnSelchangeComboState)
ON_COMMAND(ID_KEYBOARD_LEFTARROW, OnLeftArrow)
ON_COMMAND(ID_KEYBOARD_RIGHTARROW, OnRightArrow)
ON_COMMAND(ID_KEYBOARD_UPARROW, OnUpArrow)
ON_COMMAND(ID_KEYBOARD_DOWNARROW, OnDownArrow)	
	ON_UPDATE_COMMAND_UI(ID_VIEW_BOTTOMPANE, OnUpdateViewBottompane)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRootflyDlg message handlers

void CRootflyDlg::ClearSelection()
{
  m_selected_root_id = -1;
  m_selected_index = -1;
  m_over_root = false;
}

//STATUSBAR
static UINT BASED_CODE StatusBarIndicators[] =
{
	ID_INDICATOR_STATUS,
  ID_INDICATOR_PIXEL
};

BOOL CRootflyDlg::OnInitDialog()
{
  CDialog::OnInitDialog();
  // IDM_ABOUTBOX must be in the system command range.
  ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
  ASSERT(IDM_ABOUTBOX < 0xF000);
  DialogGUIstate = 0;  //Dialog size is restored size
  
  CMenu* pSysMenu = GetSystemMenu(FALSE);
  if (pSysMenu != NULL)
  {
    CString strAboutMenu;
    strAboutMenu.LoadString(IDS_ABOUTBOX);
    if (!strAboutMenu.IsEmpty())
    {
      pSysMenu->AppendMenu(MF_SEPARATOR);
      pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
    }
  }
  
  SetIcon(m_hIcon, TRUE);			// Set big icon
  SetIcon(m_hIcon, FALSE);		// Set small icon
		
  	//STATUSBAR
  Status_bar.Create(this);	
	Status_bar.SetIndicators(StatusBarIndicators,2);
  
  CRect statrect;
	GetClientRect(&statrect);

  Status_bar.SetPaneInfo(0,ID_INDICATOR_STATUS,SBPS_NORMAL,statrect.Width()-500);
  Status_bar.SetPaneInfo(1,ID_INDICATOR_PIXEL,SBPS_STRETCH,0);

  Status_bar.GetStatusBarCtrl().SetBkColor(RGB(180,180,180));

  m_controller.Attach(this);    
  SetMode(MODE_NOFILELOADED);  
  
  
  // These are all of the cursors used in Rootfly
  m_cur_openhand   = AfxGetApp()->LoadCursor(IDC_CUR_OPENHAND);
  m_cur_closedhand = AfxGetApp()->LoadCursor(IDC_CUR_CLOSEDHAND);
  m_cur_pen        = AfxGetApp()->LoadCursor(IDC_CUR_PEN);
  m_cur_uparrow    = AfxGetApp()->LoadStandardCursor(IDC_UPARROW);
  m_cur_sizeall    = AfxGetApp()->LoadStandardCursor(IDC_SIZEALL);
  m_cur_sizens     = AfxGetApp()->LoadStandardCursor(IDC_SIZENS);
  
  // No roots are selected by default
  ClearSelection();
   
  CRect section_rect;  
  m_images_section.GetWindowRect(&section_rect); 
  ScreenToClient(section_rect);  
  
  // sets up the initial custom box
  std::vector<CString> condition_set;
  condition_set = m_set_data->GetConditionSet();
  // if size is zero then there are no codes to load
  // probably first time using program
  if (condition_set.size() == 0)
  {
    InitializeConditionSet();
  } else
  {
    for (int i = 0; i < condition_set.size(); i++)
    {
      m_combo_custom.AddString(condition_set.at(i));
    }
  }
  m_combo_custom.SetCurSel(m_set_data->GetConditionDefaultIndex());
  
  // read from data structure and set check based off bool
  // if entry not present in registry, is set to on by default
  m_checkbox_show_root_ids.SetCheck(m_set_data->RootIdIsEnabled());
  m_checkbox_show_dead_roots.SetCheck(m_set_data->DeadRootsIsEnabled());
  m_checkbox_show_diameters.SetCheck(m_set_data->ShowDiametersIsEnabled());
  
  // ---------------------places toolbar in dialog------------------ //
  {
    /*if (!m_wndToolBar.Create(this)
      || !m_wndToolBar.LoadToolBar(IDR_TOOLBAR1))
    {
      TRACE0("Failed to Create Dialog Toolbar\n");
      EndDialog(IDCANCEL);
    }*/

    CRect	rcClientOld; 
    CRect	rcClientNew; 
    GetClientRect(rcClientOld); 
    RepositionBars(AFX_IDW_CONTROLBAR_FIRST,
      AFX_IDW_CONTROLBAR_LAST,0,reposQuery,rcClientNew);
    CPoint ptOffset(rcClientNew.left-rcClientOld.left,
      rcClientNew.top-rcClientOld.top);
    CRect	rcChild;
    CWnd*	pwndChild = GetWindow(GW_CHILD);
    while (pwndChild) 
    {
      pwndChild->GetWindowRect(rcChild); 
      ScreenToClient(rcChild);
      rcChild.OffsetRect(ptOffset);
      pwndChild->MoveWindow(rcChild,FALSE); 
      pwndChild = pwndChild->GetNextWindow();
    }
    CRect	rcWindow;
    GetWindowRect(rcWindow);
    rcWindow.right += rcClientOld.Width() - rcClientNew.Width();
    rcWindow.bottom += rcClientOld.Height() - rcClientNew.Height();
    RepositionBars(AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST,0);
  }
  // ---------------------end of toolbar placement------------------ //
 
  m_controller.C_SetModify(false);

  return TRUE;  // return TRUE  unless you set the focus to a control
}

void CRootflyDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
  if ((nID & 0xFFF0) == IDM_ABOUTBOX)
  {
    CAboutDlg dlgAbout;
    dlgAbout.DoModal();
  }
  else      CDialog::OnSysCommand(nID, lParam);  
}

/// DRAWING RELATED MEHODS
void CRootflyDlg::OnPaint() 
{
  if (IsIconic())
  {
    CPaintDC dc(this);     
    SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);
    
    // Center icon in client rectangle
    int cxIcon = GetSystemMetrics(SM_CXICON);
    int cyIcon = GetSystemMetrics(SM_CYICON);
    CRect rect;
    GetClientRect(&rect);
    int x = (rect.Width() - cxIcon + 1) / 2;
    int y = (rect.Height() - cyIcon + 1) / 2;
    
    dc.DrawIcon(x, y, m_hIcon);
  }
  else
  {   
    CDialog::OnPaint();
    ReDraw(1, 1);
  }
}

/*ON_INIT,ROOT_ADDED,ROOT_DELETED,ROOT_RESIZED,POINT_ADDED,POINT_REMOVED,POINT_CHANGED,
DIAMETER_ADDED,DIAMETER_REMOVED,DIAMETER_CHANGED,SESSION_CHANGED,TUBE_CHANGED,
EXPERIMENT_CHANGED,CURRENT_IMAGE_CHANGED,SESSION_DATE_CHANGED,OPERATOR_CHANGED,SAVE_STATUS_SAVED,
SAVE_STATUS_UNSAVED,FILENAME_CHANGED,IMAGEDIR_CHANGED,IMAGEFILEFORMAT_CHANGED,WINDOW_CHANGED,
COLOR_CHANGED,STATUS_CHANGED,CONDITION_CODE_ADDED,SHOW_ROOT_ID_CHANGED,SHOW_DEAD_ROOTS_CHANGED, 
SHOW_DIAMETERS_CHANGED,DRAW_SETTINGS_CHANGED,TUBE_ADDED,NEW_FILE_LOADED,FILE_SAVED,
ROOT_INSTANCE_CLEAR,ROOT_CLEAR,TUBE_CLEAR,EXPERIMENT_CLEAR,PREFERENCES_READ,PREFERENCES_SAVED
*/
void CRootflyDlg::OnUpdate(Subject* theChangeSubject, int code_update) 
{   
  CString name,name2;
  char backslash = '\\';
  int index;
  switch(code_update)
  {
  case SAVE_STATUS_SAVED:
    name = m_app_data->GetFileName();
    index = name.ReverseFind(backslash);
    name2.Format("%d", index);
    if (index == -1)
    {
      SetWindowText(name);
    } else
    {
      name = name.Right(name.GetLength()-index-1);
      SetWindowText("Rootfly - [" + name + "]");
    }
    break;
  case SAVE_STATUS_UNSAVED:
    name = m_app_data->GetFileName();
    index = name.ReverseFind(backslash);
    name2.Format("%d", index);
    if (index == -1)
    {
      SetWindowText("Rootfly - [" + name + " *]");
    } 
    else
    {
      name = name.Right(name.GetLength()-index-1);
      SetWindowText("Rootfly - [" + name + "*]");
    }   
    break; 
  case FILENAME_CHANGED:
    name = m_app_data->GetFileName();
    index = name.ReverseFind(backslash);
    name2.Format("%d", index);
    if (index == -1)
    {
      if (name != DEFAULT_FILENAME)
        SetWindowText("Rootfly - [" + name + " *]");
      else
        SetWindowText("Rootfly - [" + name + "]");
    }
    else
    {
      name = name.Right(name.GetLength()-index-1);
      if (name != DEFAULT_FILENAME)
        SetWindowText("Rootfly - [" + name + " *]");
      else
        SetWindowText("Rootfly - [" + name + "]");
    }   
    break;     
  default: break;
  }
}

// handles the code to display the properties about the selected root
void CRootflyDlg::UpdateProperties(int selected_root)
{
  // no root selected go back to initial conditions
  if (selected_root == -1)
  {
    m_edit_length.SetWindowText("");
    m_edit_rootid.SetWindowText("");
    m_edit_diameter.SetWindowText("");
    m_combo_color.SetWindowText("");
    m_combo_state.SetWindowText("");
    m_combo_custom.SetWindowText("");
    return;
  }
  
  const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
  assert(tube);  if (!tube)  return;
  const Root *root = tube->GetRoot( selected_root );
  assert(root);  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(m_app_data->GetSession());
  if (!inst)  return;
  const std::vector<RPoint> *line = inst->GetAllPoints();
  assert(line);  if (!line)  return;
  CString tempstr;
  
  // displays the root id, checks that the current window is the same as the primary window for the RootInstance, if it is not
  // then it will show the RootInstance as "WindowID(PrimaryWindow
  if (m_app_data->GetWindow() != root->GetPrimaryWindow()) tempstr.Format("%d(%d)",root->GetWindowID(),root->GetPrimaryWindow());
  else tempstr.Format("%d",root->GetWindowID());
  
  m_edit_rootid.SetWindowText(tempstr);
  
  // updates the length of the root
  double templength = 0;
  double dx, dy;
  const std::vector<RPoint> *points= inst->GetAllPoints();
  for (int j=1; j<points->size(); j++)
  {
    dx = (points->at(j).x-points->at(j-1).x) / m_exp_data->GetMMConversion();
    dy = (points->at(j).y-points->at(j-1).y) / m_exp_data->GetMMConversion();
    templength += sqrt(pow(dx,2)+pow(dy,2));
  }
  tempstr.Format("%.2f mm",templength);
  m_edit_length.SetWindowText(tempstr);
  
  // displays the diameter
  const RDiameter *diam = inst->GetDiameter(0);
  double rad = diam->radius / m_exp_data->GetMMConversion();
  tempstr.Format("%.2f mm",rad*2);
  m_edit_diameter.SetWindowText(tempstr); 
  
  // displays the state of the root
  if (inst->IsAlive()) m_combo_state.SetCurSel(1);
  else m_combo_state.SetCurSel(0);
  
  // displays the root color (white, black, dark brown, or light brown)
  int color = inst->GetColor();
  assert(color >= 0 && color <= 3);
  if (color < 0 || color > 3)  color = 0;
  m_combo_color.SetCurSel( color );
  
  // custom box
  CString user_code = inst->GetConditionCode();
  // code is no longer valid reset condition back to default
  if (m_combo_custom.SelectString(-1,user_code) == CB_ERR)
  {
    CString temp;
    m_combo_custom.GetLBText(m_set_data->GetConditionDefaultIndex(), temp);
    m_controller.C_SetConditionCode(selected_root,temp);
    m_combo_custom.SetCurSel(m_set_data->GetConditionDefaultIndex());
  }
} 

// returns the rect (in client coordinates) with associated index and type
// index 0 is the middle rect
// if type is 0 then returns a window rect
// if type is 1 then returns a session rect
CRect CRootflyDlg::GetThumbnailRect(int index,int type)
{
  CRect rect;
  if (type == 0)
  {
    switch(index+((NUM_THUMBNAILS-1)/2))
    {
    case 0: m_Window_Pic_1.GetWindowRect(&rect); break;
    case 1: m_Window_Pic_2.GetWindowRect(&rect); break;
    case 2: m_Window_Pic_3.GetWindowRect(&rect);  break;
    case 3: m_Window_Pic_4.GetWindowRect(&rect);  break;
    case 4: m_Window_Pic_5.GetWindowRect(&rect);  break;
    default:  m_Window_Pic_3.GetWindowRect(&rect);
    }
  } else if (type == 1)
  {
    switch(index+((NUM_THUMBNAILS-1)/2))
    {
    case 0: m_Session_Pic_1.GetWindowRect(&rect); break;
    case 1: m_Session_Pic_2.GetWindowRect(&rect); break;
    case 2: m_Session_Pic_3.GetWindowRect(&rect); break;
    case 3: m_Session_Pic_4.GetWindowRect(&rect); break;
    case 4: m_Session_Pic_5.GetWindowRect(&rect); break;
    default:  m_Session_Pic_3.GetWindowRect(&rect);
    }
  }
  ScreenToClient(rect);
  return rect;
}

// draws the images in the main window
bool CRootflyDlg::DrawImage(blepo::ImgBgr img, CRect rect, int selected, CDC& dc) const
{
  //  ImageInfo current_image;
  ImageFileFormat ii = m_controller.C_GetImageFileFormat();
  CString temp_str;
  
  // checks to see if file is valid
  // draws image if valid, displays message if image is unavailable
  temp_str = ii.GetFileName(m_exp_data->GetImageDir(),m_app_data->GetTube(),m_app_data->GetWindow()+selected,m_app_data->GetSession());
  if (temp_str != "") 
  {
    blepo::Draw(img, dc, CRect(0, 0, img.Width(), img.Height()), rect); 
    return true;
  }
  else
  {
    CRect section_rect = GetClientRectOfImageDisplayArea();
    rect.top += rect.Height()/2;  // for some reason, DT_VCENTER is not working
    if (GetMode() != MODE_NOFILELOADED) dc.DrawText("No Image Available", rect, DT_CENTER);
    return false;
  }
}

// displays one or three images, depending upon whether in zoomed out mode
void CRootflyDlg::UpdateMainImage(CDC& dc)
{
  if ( GetMode() == MODE_NOFILELOADED) return;
  m_graphics.FillImageDisplayAreaWithGray(dc,GetClientRectOfImageDisplayArea());
  SetZoomWindows();  // needed in case image aspect ratio has changed
  
  CRect rect, rect1, rect2, rect3 ;
  blepo::ImgBgr img, img1, img2, img3;    
  
  img  = m_app_data->GetWindowImage(2);
  img1 = m_app_data->GetWindowImage(1);
  img2 = m_app_data->GetWindowImage(2);
  img3 = m_app_data->GetWindowImage(3);
  
  rect.SetRect(0,0,img.Width(),img.Height());
  {
    {
      bool null1 = (img1.Width() == 0 || img1.Height() == 0);
      bool null3 = (img3.Width() == 0 || img3.Height() == 0);
      bool samesize1 = (img.Width () == img1.Width () && img.Height() == img1.Height());
      bool samesize3 = (img.Width () == img3.Width () && img.Height() == img3.Height());
      if ( (!null1 && !samesize1) || (!null3 && !samesize3))
      {
        assert(0);  // if images are of different sizes, we might have problems with preserving the aspect ratio in the drawing, although it will not likely cause huge errors -- STB
      }
    }
    m_converter.SetImageSize( CSize(img.Width(),img.Height()) );
  }
  
  // display image(s)
  if ( IsZoomOutMode() )
  {
    int w = m_app_data->GetWindow();
    rect1 = m_converter.GetClientRect( w-1 );
    rect2 = m_converter.GetClientRect( w   );
    rect3 = m_converter.GetClientRect( w+1 );
    
    DrawImage(img1, rect1, -1, dc);
    if ( GetMode() == MODE_NOIMAGEDISPLAYED )
    {
      if (DrawImage(img2, rect2, 0, dc))
      {
        SetMode(MODE_IMAGEDISPLAYED);       
      }
    }
    else if (!DrawImage(img2, rect2, 0, dc))
    {
      SetMode(MODE_NOIMAGEDISPLAYED);       
    }
    DrawImage(img3, rect3, 1, dc);
  }
  else
  {     
    int w = m_app_data->GetWindow();
    rect = m_converter.GetClientRect( w );
    if ( GetMode() == MODE_NOIMAGEDISPLAYED )
    {
      if (DrawImage(img, rect, 0, dc))
      {
        SetMode(MODE_IMAGEDISPLAYED);        
      }
    }
    else if (!DrawImage(img, rect, 0, dc)) 
    {
      SetMode(MODE_NOIMAGEDISPLAYED);      
    }
  }
}

void CRootflyDlg::UpdateThumbnails()
{
  blepo::ImgBgr img;
  CClientDC dc(this);
  ImageInfo current_image;
  ImageFileFormat ii = m_controller.C_GetImageFileFormat();
  CRect rect;
  int index, current, max, pos;
  CString temp_str;
  CPoint corner;
  
  // used for coloring
  COLORREF COLOR_BLUE =        RGB(0,0,255);
  COLORREF COLOR_BLACK =       RGB(0,0,0);
  COLORREF COLOR_SYS_GRAY =    ::GetSysColor(COLOR_3DFACE);
  CBrush black_brush(COLOR_BLACK);
  CBrush blue_brush(COLOR_BLUE);
  CBrush background_brush(COLOR_SYS_GRAY);
  
  if (GetMode() != MODE_NOFILELOADED)
  {
    // draws window thumbnails then session thumbnails
    for (pos = 0; pos < 2; pos++)
    {
      // draws a set of five thumbnails
      for (index = -((NUM_THUMBNAILS-1)/2); index < ((NUM_THUMBNAILS+1)/2); index++)
      {
        rect = GetThumbnailRect(index,pos);
        
        // tests to see if there is a file for the thumbnail and sets the current and max varibles
        if (pos == 0)
        {
          img = m_app_data->GetWindowImage(index+2);
          temp_str = ii.GetFileName(m_exp_data->GetImageDir(),m_app_data->GetTube(),m_app_data->GetWindow()+index,m_app_data->GetSession());
          current = index  + m_app_data->GetWindow();
          max = m_exp_data->GetMaxWindows();
         
        } else
        {
          img = m_app_data->GetSessionImage(index+2);
          temp_str = ii.GetFileName(m_exp_data->GetImageDir(),m_app_data->GetTube(),m_app_data->GetWindow(),m_app_data->GetSession()+index);
          current = index + m_app_data->GetSession();
          max = m_exp_data->GetMaxSessions();
          
        }
        CRect temp(0, 0, img.Width(), img.Height());
        
        // draws the image
        if (temp_str != "")
          blepo::Draw(img, dc, temp, rect);
        else dc.FillRect(rect,&background_brush);
        
        // places index of thumbnail in corner
        if ((current> 0) && (current <= max))
        {
          corner = rect.TopLeft();
          temp_str.Format("%d", current);
          dc.TextOut(corner.x+1, corner.y+1, temp_str);
        }
        
        // frames the rect
        if (index == 0)
        {
          dc.FrameRect(rect,&blue_brush);
          for(int i = 0; i < SELECTED_THUMBNAIL_BORDER_WIDTH; i++)
          {
            rect.InflateRect(1,1);
            dc.FrameRect(rect,&blue_brush);
          }
        }
        else dc.FrameRect(rect,&black_brush); 
        
      }    
    }
    UpdateSessionDateText();
  }
}

// updates the session dates under the thumbnails
void CRootflyDlg::UpdateSessionDateText()
{  
  // updates dates
  Date temp_date;
  CString temp;
  
  temp_date = m_exp_data->GetSessionDate(m_app_data->GetSession()-2);            
  if (temp_date.month != -1) m_session_date1.SetWindowText(SessionDatesDlg::GetStringFromDate(temp_date));
  else m_session_date1.SetWindowText("");
  
  temp_date = m_exp_data->GetSessionDate(m_app_data->GetSession()-1);
  if (temp_date.month != -1) m_session_date2.SetWindowText(SessionDatesDlg::GetStringFromDate(temp_date));
  else m_session_date2.SetWindowText("");
  
  temp_date = m_exp_data->GetSessionDate(m_app_data->GetSession());
  if (temp_date.month != -1) m_session_date3.SetWindowText(SessionDatesDlg::GetStringFromDate(temp_date));
  else m_session_date3.SetWindowText("");
  
  temp_date = m_exp_data->GetSessionDate(m_app_data->GetSession()+1);
  if (temp_date.month != -1) m_session_date4.SetWindowText(SessionDatesDlg::GetStringFromDate(temp_date));
  else m_session_date4.SetWindowText("");
  
  temp_date = m_exp_data->GetSessionDate(m_app_data->GetSession()+2);
  if (temp_date.month != -1) m_session_date5.SetWindowText(SessionDatesDlg::GetStringFromDate(temp_date));
  else m_session_date5.SetWindowText("");
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CRootflyDlg::OnQueryDragIcon()
{
  return (HCURSOR) m_hIcon;
}

void CRootflyDlg::OnBUTTONNewRoot() 
{
  if ( GetMode() == MODE_NOIMAGEDISPLAYED ) return;
  m_selected_root_id = -1;
  SetMode(MODE_DRAWINGNEWROOT);
  m_current_root_being_drawn.ClearAll();
  ReDraw(1,0);
}

/**
/ @brief: This function fills the member root with the 
/         selected root's information.  Then the mode
/         changes to MODE_EXTENDINGENDPOINTS until
/         right button is clicked to abort, or a valid 
/         point is placed on the drawing area.
*/
void CRootflyDlg::OnBUTTONExtendEndpoints() 
{
  if ( GetMode() != MODE_ROOTSELECTED )  return;  
  const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
  assert(tube);
  if (!tube)  return;
  const vector<Root> *root = tube->GetAllRoots();
  if (!root) return;
  m_current_root_being_drawn.ClearAll();
  for ( int i = 0; i <  root->size(); i++ )
    if ( root->at(i).GetID() == m_selected_root_id )
    {
      const RootInstance *inst = root->at(i).GetRootInstance(m_app_data->GetSession());  
      if (!inst) return;
      if (inst->IsAlive()) m_current_root_being_drawn.SetAlive();
      else m_current_root_being_drawn.SetDead();
      m_current_root_being_drawn.SetSessionNumber(m_app_data->GetSession());
      m_current_root_being_drawn.SetConditionCode(inst->GetConditionCode());
      m_current_root_being_drawn.SetColor(inst->GetColor());
      const vector<RPoint> *points = inst->GetAllPoints();
      assert(points);
      if (!points)  return;
      for ( int j = 0; j < points->size(); j++ )
      {
        RPoint *temp = m_current_root_being_drawn.CreatePoint();        
        temp->x = points->at(j).x;
        temp->y = points->at(j).y;
        temp->window = points->at(j).window;        
      }
      RDiameter *temp_diam = m_current_root_being_drawn.CreateDiameter();
      temp_diam->r_point = inst->GetDiameter(0)->r_point;
      temp_diam->radius = inst->GetDiameter(0)->radius;
    }    
    SetMode(MODE_EXTENDINGENDPOINT); 
}

void CRootflyDlg::OnBUTTONDeleteRoot() 
{
  const Experiment* exp = m_rfy_data->GetExperiment();   
  const Tube *tube = exp->GetTube(m_app_data->GetTube());
  if (tube == NULL) return;
  
  if (m_selected_root_id == -1)
  {
    // This should not happen, because 'delete root' button is disabled if no root selected
    AfxMessageBox("Please Select a Root First");
    return;
  }
  const Root *TempRoot = tube->GetRoot(m_selected_root_id);
  
  CString temp_str;
  temp_str.Format("Delete Root %d?", TempRoot->GetWindowID());
  
  if (AfxMessageBox(temp_str, MB_YESNO) == IDYES)
  {
    m_controller.C_DeleteRoot(m_selected_root_id);
    m_selected_root_id = -1;
    SetMode(MODE_IMAGEDISPLAYED);
    ReDraw(1,0);
  }
}

/**
* This function is called when the user clicks the Copy... button
*/
void CRootflyDlg::OnBUTTONCopySession() 
{  
  CCopySession dlg;
  
  int curr_session = m_app_data->GetSession();
  int curr_window = m_app_data->GetWindow();  
  
  //Set the initial source and destination sessions in the copy session dialog.
  //  If the user is on the first session, set the source to 1 and destination to 2.
  //  otherwise, set the destination to the current session, and the source to the session right before it.
  if (curr_session > 1)
  {
    dlg.SetSource(curr_session - 1);
    dlg.SetDestination(curr_session);
  }
  else 
  {
    dlg.SetSource(1);
    dlg.SetDestination(2);
  }
  
  int response = dlg.DoModal();
  
  //If the user selected OK in the copy session dialog...
  if (response == IDOK) 
  {
    if (dlg.GetSource() >= dlg.GetDestination()) 
    {
      AfxMessageBox("Destination session must be greater than source session.  Unable to copy.");
      return;
    }
    
    //To copy from all the tubes and all the windows
    CString tubename = m_app_data->GetTube();
    
    //NOTE:  If copying roots in ALL windows, from a given session to a given session, 
    //        pass -1 as the window parameter to C_CopySession
    if (dlg.GetCopyAll()) 
    {
      
      // copy all tubes (this is the option displayed to user, so we had better do it)
      const vector<Tube> *tubes = m_exp_data->GetAllTubes();
      m_controller.C_CopySession(dlg.GetSource(), dlg.GetDestination(), tubename, -1);
  
      
    } 
    else 
    {
      m_controller.C_CopySession(dlg.GetSource(), dlg.GetDestination(), tubename, curr_window);
    }
    
    ReDraw(1, 0);
  }
}

/**
/
/    @action: This resizes the individual items in the main window
/     and checks the menu to see if any objects should be hidden
/
*/  
void CRootflyDlg::OnSizeNew(UINT nType, int cx, int cy)
{
  Rootfly_Left_Objects_Hide();
  Rootfly_Bottom_Objects_Hide();
  Rootfly_Right_Objects_Hide();

  m_window_section.SetWindowPos(&m_window_section,19,29,167,530,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_images_section.SetWindowPos(&m_images_section,191,29-19,cx-369,cy-176,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_session_section.SetWindowPos(&m_session_section,192,cy-146-19,628,141,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_properties_section.SetWindowPos(&m_properties_section,cx-173,135,163,200,SWP_NOZORDER|SWP_SHOWWINDOW);
 // m_status_text.SetWindowPos(&m_status_text,160,12,cx-160-20,20,SWP_NOZORDER|SWP_SHOWWINDOW);
    
  //Tube, Window, Session
  
  m_spin_tube.SetWindowPos(&m_spin_tube,cx-36,41,16,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_edit_tube.SetWindowPos(&m_edit_tube,cx-108,43,70,21,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_tube_text.SetWindowPos(&m_tube_text,cx-167,47,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);  
  m_spin_window.SetWindowPos(&m_spin_window,cx-36,70,16,23, SWP_NOZORDER|SWP_SHOWWINDOW);
  m_edit_window.SetWindowPos(&m_edit_window,cx-108,72,70,21,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_window_text.SetWindowPos(&m_window_text,cx-167,76,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);  
  m_spin_session.SetWindowPos(&m_spin_session,cx-36,100,16,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_edit_session.SetWindowPos(&m_edit_session,cx-108,102,70,21,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_session_text.SetWindowPos(&m_session_text,cx-167,103,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);  
  
  //Properties Section
  
  m_root_id_text.SetWindowPos(&m_root_id_text,cx-165,158,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_edit_rootid.SetWindowPos(&m_edit_rootid,cx-108,155,88,22,SWP_NOZORDER|SWP_SHOWWINDOW);   
  m_diameter_text.SetWindowPos(&m_diameter_text,cx-165,188,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_edit_diameter.SetWindowPos(&m_edit_diameter,cx-108,185,88,22,SWP_NOZORDER|SWP_SHOWWINDOW);   
  m_length_text.SetWindowPos(&m_length_text,cx-165,217,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_edit_length.SetWindowPos(&m_edit_length,cx-108,215,88,22,SWP_NOZORDER|SWP_SHOWWINDOW); 
  m_color_text.SetWindowPos(&m_color_text,cx-165,247,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_combo_color.SetWindowPos(&m_combo_color,cx-108,245,88,80,SWP_NOZORDER|SWP_SHOWWINDOW);  
  m_state_text.SetWindowPos(&m_state_text,cx-165,277,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_combo_state.SetWindowPos(&m_combo_state,cx-108,275,88,80,SWP_NOZORDER|SWP_SHOWWINDOW);  
  m_custom_text.SetWindowPos(&m_custom_text,cx-165,307,50,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_combo_custom.SetWindowPos(&m_combo_custom,cx-108,305,88,100,SWP_NOZORDER|SWP_SHOWWINDOW);
  
  
  //Checkboxes
  
  m_checkbox_show_diameters.SetWindowPos(&m_checkbox_show_diameters,cx-167,342,119,14,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_checkbox_show_dead_roots.SetWindowPos(&m_checkbox_show_dead_roots,cx-167,367,119,14,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_checkbox_show_root_ids.SetWindowPos(&m_checkbox_show_root_ids,cx-167,392,119,14,SWP_NOZORDER|SWP_SHOWWINDOW);
  
  //Buttons
  
  m_button_new_root.SetWindowPos(&m_button_new_root,cx-167,417,89,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_button_extend_endpoints.SetWindowPos(&m_button_extend_endpoints,cx-167,447,89,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_button_move_all_roots.SetWindowPos(&m_button_move_all_roots,cx-167,477,89,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_button_delete_root.SetWindowPos(&m_button_delete_root,cx-167,507,89,23,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_button_copy.SetWindowPos(&m_button_copy,cx-167,537,89,23,SWP_NOZORDER|SWP_SHOWWINDOW);    
  m_check_zoom_out.SetWindowPos(&m_check_zoom_out,cx-167,587,89,23,SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW);
  GetDlgItem(IDC_BUTTON_Auto_Detect)->ShowWindow (SW_SHOW); //Temp fix, autodetect group needs to add function for redrawing of button
  

  //Window Thumbnails
  m_Window_Pic_1.SetWindowPos(&m_Window_Pic_1,32,53,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Window_Pic_2.SetWindowPos(&m_Window_Pic_2,32,153,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Window_Pic_3.SetWindowPos(&m_Window_Pic_3,32,253,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Window_Pic_4.SetWindowPos(&m_Window_Pic_4,32,353,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Window_Pic_5.SetWindowPos(&m_Window_Pic_5,32,453,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_scrollbar_vertical.SetWindowPos(&m_scrollbar_vertical,160,43,15,509,SWP_NOZORDER|SWP_SHOWWINDOW);  
  
  //Session Thumbnails
  m_Session_Pic_1.SetWindowPos(&m_Session_Pic_1,206,cy-129-19,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Session_Pic_2.SetWindowPos(&m_Session_Pic_2,328,cy-129-19,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Session_Pic_3.SetWindowPos(&m_Session_Pic_3,450,cy-129-19,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Session_Pic_4.SetWindowPos(&m_Session_Pic_4,572,cy-129-19,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_Session_Pic_5.SetWindowPos(&m_Session_Pic_5,694,cy-129-19,113,85,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_scrollbar_horizontal.SetWindowPos(&m_scrollbar_horizontal,198,cy-20-19,618,15,SWP_NOZORDER|SWP_SHOWWINDOW);  
  
  //Session Dates
  m_session_date1.SetWindowPos(&m_session_date1,236,cy-40-19,70,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_session_date2.SetWindowPos(&m_session_date2,358,cy-40-19,70,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_session_date3.SetWindowPos(&m_session_date3,480,cy-40-19,70,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_session_date4.SetWindowPos(&m_session_date4,602,cy-40-19,70,15,SWP_NOZORDER|SWP_SHOWWINDOW);
  m_session_date5.SetWindowPos(&m_session_date5,724,cy-40-19,70,15,SWP_NOZORDER|SWP_SHOWWINDOW); 
  

  UINT m_bottomState = GetMenu()->GetMenuState(ID_VIEW_BOTTOMPANE, MF_BYCOMMAND);
  if(m_bottomState == MF_UNCHECKED){
    int m_sessionSectionBottom;
    RECT m_RectWindow;
    m_session_section.GetWindowRect(&m_RectWindow);
	m_sessionSectionBottom = m_RectWindow.bottom;
    m_images_section.GetWindowRect(&m_RectWindow);
    m_images_section.SetWindowPos(&m_images_section, 191,29-19, m_RectWindow.right - m_RectWindow.left , m_sessionSectionBottom - m_RectWindow.top, SWP_NOZORDER|SWP_SHOWWINDOW);
	Rootfly_Bottom_Objects_Hide();
  }

  UINT m_rightState = GetMenu()->GetMenuState(ID_VIEW_RIGHTPANE, MF_BYCOMMAND);
  if(m_rightState == MF_UNCHECKED){
    int m_propertiesSectionRight;
    RECT m_RectWindow;
    m_properties_section.GetWindowRect(&m_RectWindow);
	m_propertiesSectionRight = m_RectWindow.right;
    m_images_section.GetWindowRect(&m_RectWindow);
    m_images_section.SetWindowPos(&m_images_section, 191,29-19, m_propertiesSectionRight - m_RectWindow.left , m_RectWindow.bottom - m_RectWindow.top, SWP_NOZORDER|SWP_SHOWWINDOW);
	Rootfly_Right_Objects_Hide();
  }

  UINT m_leftState = GetMenu()->GetMenuState(ID_VIEW_LEFTPANE, MF_BYCOMMAND);
  if(m_leftState == MF_UNCHECKED){
    int m_windowSectionLeft;
    int m_windowSectionTop;
	int m_imagesSectionLength;
    RECT m_RectWindow;
    m_window_section.GetWindowRect(&m_RectWindow);
    m_windowSectionLeft = m_RectWindow.left;
    m_windowSectionTop = m_RectWindow.top;
	m_images_section.GetClientRect(&m_RectWindow);
	m_imagesSectionLength = m_RectWindow.bottom;
    m_images_section.GetWindowRect(&m_RectWindow);
    m_images_section.SetWindowPos(&m_images_section, 19,29-19, m_RectWindow.right - m_windowSectionLeft , m_imagesSectionLength, SWP_NOZORDER|SWP_SHOWWINDOW);
	Rootfly_Left_Objects_Hide();
  }

}

void CRootflyDlg::OnSize(UINT nType, int cx, int cy) 
{
//  m_window_size = CPoint( cx, cy );
  
  /* working on the resizing problems
  // prevents items from resizeing if size is less than these values
  if (cy < 595) 
  {
  SetWindowPos(this,0,0,cx,595,SWP_NOZORDER|SWP_SHOWWINDOW);
  return;
  } else if (cx < 990) 
  {
  SetWindowPos(this,0,0,,990,cy,SWP_NOZORDER|SWP_SHOWWINDOW);
  return;
}  */
  //Sections
    //Sections
  
  int  a,b,c,d;
	char aa[10],cc[10],bb[10],dd[10],xx[10],yy[10];
	CString MFCString, A, B, C, D, X, Y;
	
  	
	CRect section_rect2;
	m_images_section.GetWindowRect(&section_rect2); 
	
		a = section_rect2.left;
		b = section_rect2.right;
		c = section_rect2.top;
		d = section_rect2.bottom;
	 
		//convert ints to chars
		itoa(a,aa,10);
		itoa(b,bb,10);
		itoa(c,cc,10);
		itoa(d,dd,10);
		itoa(cx,xx,10);
		itoa(cy,yy,10);
		//converts chars to strings
		A = aa;
		B = bb;
		C = cc;
		D = dd;
    X = xx;
    Y = yy;
		//Concat
		MFCString = A + " " +B + " "+ C + " " +D + " " + X+ " " + Y;
	
		//Display Message box
	//AfxMessageBox(MFCString);]

    //nType == SIZE_MAXIMIZED
    //nType == SIZE_RESTORED
		OnSizeNew(nType,cx, cy);




  SetZoomWindows();  
  UpdateStatusText();

}

CRect CRootflyDlg::GetClientRectOfImageDisplayArea() const
{
  CRect section_rect;
  m_images_section.GetWindowRect(&section_rect);
  
  section_rect.DeflateRect(1,7,2,2);
  ScreenToClient(section_rect); 
  return section_rect;
}

/* 
/  @brief:  This calculates the window position and corresonding
/           boundary rectangles for the drawing areas depending on
/           the zoom state and the current window size.
*/
void CRootflyDlg::SetZoomWindows()
{
  CRect main_rect, top_rect, middle_rect, bottom_rect, section_rect, clip_rect;
  blepo::ImgBgr img;
  double main_img_asp_ratio, new_rect_width, new_rect_height;
  section_rect = GetClientRectOfImageDisplayArea();
  
  img = m_app_data->GetWindowImage(2);
  main_img_asp_ratio = ((double) img.Width()) / ((double) img.Height());
  CPoint cpt = section_rect.CenterPoint();
  int w = section_rect.Width(), h = section_rect.Height();
  if (IsZoomOutMode())
  {
    new_rect_height = h / 3;
    new_rect_width = new_rect_height * main_img_asp_ratio;
  }
  else
  {
    double ww = h * main_img_asp_ratio, hh = w / main_img_asp_ratio;
    new_rect_width  = (ww > w) ? w  : ww;  // set width and height whether image is wide or tall
    new_rect_height = (ww > w) ? hh : h ;
  }
  main_rect.left   = cpt.x - (int) (new_rect_width  / 2);
  main_rect.top    = cpt.y - (int) (new_rect_height / 2);
  main_rect.right  = cpt.x + (int) (new_rect_width  / 2);
  main_rect.bottom = cpt.y + (int) (new_rect_height / 2);
  
  m_converter.SetClientRect( main_rect );
  m_converter.SetWindow( m_app_data->GetWindow() );  // may be unnecessary, but will not hurt
}

void CRootflyDlg::OnEditOptions() 
{	
  OptionsDlg optionsdlg(&m_image_file_format,this);
  int id = optionsdlg.DoModal();
  if (id == IDOK)
  {
    m_controller.C_SetImageFileFormat(m_image_file_format);
    // Get the first image and get the size of the image to set the image
    // rectangle for the image converter
    CString firstImage = m_image_file_format.GetFirstImage(m_exp_data->GetImageDir());
    CRect rect = m_image_file_format.GetImageRectangle(firstImage);
    m_converter.SetImageSize( CSize(rect.Width(), rect.Height()) );
  }
}

void CRootflyDlg::OnHelpAbout() 
{
  CDialog dlg(IDD_ABOUTBOX);
  dlg.DoModal();
}

void CRootflyDlg::OnHelpContents() 
{
  // Execute Help File
  ShellExecute(NULL, "open","Rootfly.chm", NULL, NULL, SW_SHOWNORMAL);
}

void CRootflyDlg::OnOK() 
{
  if (GetFocus() == &m_edit_tube)
    OnKillfocusEditTube();
  else if (GetFocus() == &m_edit_window)
    OnKillfocusEditWindow();
  else if (GetFocus() == &m_edit_session)
    OnKillfocusEditSession();
  else if (GetFocus() == &m_edit_diameter)
    OnKillfocusEditDiameter();
  else if (GetFocus() == m_combo_custom.GetWindow(GW_CHILD))  // focus is on the child of the combo box
    OnKillfocusComboCustom();
}		

void CRootflyDlg::OnCancel() 
{
  // need to do nothing, so that 'escape' key will not cause program to exit
  // should look into this further -- STB
  CDialog::OnCancel();
}

// Set the diameter of the selected root based upon the text typed in the edit box
void CRootflyDlg::OnKillfocusEditDiameter() 
{
  if ( GetMode() == MODE_ROOTSELECTED)
		{
    const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
    if (!tube)  return;
    const Root *root = tube->GetRoot(m_selected_root_id);
    if (!root)  return;
    const RootInstance *inst = root->GetRootInstance(m_app_data->GetSession());
    if (!inst)  return;
    const RDiameter *diam = inst->GetDiameter(0);
    if (!diam)  return;
    
    CString temp;
    m_edit_diameter.GetWindowText(temp);
    double new_radius = atof(temp) * m_exp_data->GetMMConversion() / 2.0;
    if (new_radius > 0)
    {
      m_controller.C_ResizeDiameter(0, m_selected_root_id, new_radius);
    }
    ReDraw(1,0);  
  }
}

// Set the current session based upon the text typed in the edit box
// MODEIMAGEDISPLAYED and m_selected_root_id = -1 have been placed here
// to catch the case when the user selects a root and then changes the 
// image.
void CRootflyDlg::OnKillfocusEditSession() 
{
  if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
  m_selected_root_id = -1;
  CString temp;
  int session;
  // get edit box data - stores as session to send
  m_edit_session.GetWindowText(temp);
  session = atoi(temp);
  if (session < 1) 
  {
    temp.Format("%d", m_app_data->GetSession());
    m_edit_session.SetWindowText(temp);
    return;
  }
  if (session != m_app_data->GetSession())
  {
    // user enter value bigger than range. set new range
    if (session > m_exp_data->GetMaxSessions()) 
    {
      m_controller.C_SetMaxSessions(session); 
      m_spin_session.SetRange(1,session);
      m_scrollbar_horizontal.SetScrollRange(1,session);
    }
    m_scrollbar_horizontal.SetScrollPos(session);  
    m_controller.C_SetSession(session);
    temp.Format("%d", m_app_data->GetSession());
    m_edit_session.SetWindowText(temp);
    ReDraw(1, 1);   
  }
}

// Set the current window based upon the text typed in the edit box
// MODEIMAGEDISPLAYED and m_selected_root_id = -1 have been placed here
// to catch the case when the user selects a root and then changes the 
// image.
void CRootflyDlg::OnKillfocusEditWindow() 
{
  if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
  m_selected_root_id = -1;
  CString temp;
  int window;	
  
  // get edit box data - stores as window to send
  m_edit_window.GetWindowText(temp);
  window = atoi(temp);
  if (window < 1) 
  {
    temp.Format("%d", m_app_data->GetWindow());
    m_edit_window.SetWindowText(temp);
    return;  
  }
  if (window != m_app_data->GetWindow())
  {
    // user enter value bigger than range. set new range
    if (window > m_exp_data->GetMaxWindows()) 
    {
      m_controller.C_SetMaxWindows(window); 
      m_spin_window.SetRange(1,window);
      m_scrollbar_vertical.SetScrollRange(1,window);
    }
    m_scrollbar_vertical.SetScrollPos(window);
    m_controller.C_SetWindow(window);		
    m_converter.SetWindow( m_app_data->GetWindow() );
    temp.Format("%d", m_app_data->GetWindow());
    m_edit_window.SetWindowText(temp);
    ReDraw(1, 1);
  }  
}

CString CRootflyDlg::ConvertTubeName(CString input)
{
  // file_info is used to find length of tube in file format
  const vector<OrderElement> file_info = m_image_file_format.GetImageFileFormatInfo().InfoOrder;
  int tube_length;
  int input_length;             // length of string that the user entered
  
  // sets the length of the input
  input_length = input.GetLength();
  
  // find element in file
  for (int i = 0; i < file_info.size(); i++)
  {
    // sets the length of tube_length
    if (file_info.at(i).element == TUBE)
    {
      tube_length = file_info.at(i).length;
    }
  }
  
  if (input_length < tube_length)
  {
    for (int i = 0; i < tube_length-input_length; i++)
    {
      input = "0" + input;
    }
  }
  return input;
}

CString CRootflyDlg::StripPaddedZeros(CString input)
{
  int i;
  CString temp;
  for (i = 0; i < input.GetLength(); i++)
  {
    if (input.GetAt(i) != '0') break;
  }
  temp = input.Right(input.GetLength() - i);
  return temp;
}

// Set the current tube based upon the text typed in the edit box
// MODEIMAGEDISPLAYED and m_selected_root_id = -1 have been placed here
// to catch the case when the user selects a root and then changes the 
// image.
void CRootflyDlg::OnKillfocusEditTube() 
{
  if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
  m_selected_root_id = -1;
  CString temp, converted_tube;
  
  // get edit box data - stores as tube to send
  m_edit_tube.GetWindowText(temp);
  converted_tube = ConvertTubeName(temp);
  
  if (m_app_data->GetTube().Compare(temp))
  {
    m_controller.C_SetTube(converted_tube);
    // Displays the tube without padded zeros
    m_edit_tube.SetWindowText(StripPaddedZeros(temp));
    ReDraw(1, 1);
  }
}

// Set the current tube based upon the up/down spin buttons
// MODEIMAGEDISPLAYED and m_selected_root_id = -1 have been placed here
// to catch the case when the user selects a root and then changes the 
// image.
void CRootflyDlg::OnDeltaposSpinTube(NMHDR* pNMHDR, LRESULT* pResult) 
{
  if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
  m_selected_root_id = -1;
  NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
  Tag tube_tag;
  CString temp, converted_tube;
  
  m_edit_tube.GetWindowText(temp);
  converted_tube = ConvertTubeName(temp);  
  tube_tag.SetTag(converted_tube);
  if (pNMUpDown->iDelta > 0){
    for (int i = 0; i < pNMUpDown->iDelta; i++)
      tube_tag--;
  } else
  {
    for (int i = pNMUpDown->iDelta; i < 0; i++)
      tube_tag++;
  }  
  if ((tube_tag.GetTag()).Compare(m_app_data->GetTube()))
  {
    // displays the tube without padded zeros
    m_edit_tube.SetWindowText(StripPaddedZeros(tube_tag.GetTag()));
    m_controller.C_SetTube(tube_tag.GetTag());
    *pResult = 1;
    ReDraw(1, 1);
  }
  else
  {
    *pResult = 0;
  }
}

// Set the current session based upon the up/down spin buttons
// MODEIMAGEDISPLAYED and m_selected_root_id = -1 have been placed here
// to catch the case when the user selects a root and then changes the 
// image.
void CRootflyDlg::OnDeltaposSpinSession(NMHDR* pNMHDR, LRESULT* pResult) 
{  
  if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
  m_selected_root_id = -1;
  NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
  CString temp;
  int number =pNMUpDown->iPos + pNMUpDown->iDelta;
  if ((number <= 0) || (number > m_exp_data->GetMaxSessions()))
  {	
    *pResult = 1;
    return;
  }
  if (number != m_app_data->GetSession())
  {
    *pResult = 1;
    temp.Format("%d", number);
    m_edit_session.SetWindowText(temp);
    m_scrollbar_horizontal.SetScrollPos(number);
    m_controller.C_SetSession(number);
    ReDraw(1, 1);
  }
  else
  {
    *pResult = 0;
  }
}

// Set the current window based upon the up/down spin buttons
// MODEIMAGEDISPLAYED and m_selected_root_id = -1 have been placed here
// to catch the case when the user selects a root and then changes the 
// image.
void CRootflyDlg::OnDeltaposSpinWindow(NMHDR* pNMHDR, LRESULT* pResult) 
{
  if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
  m_selected_root_id = -1;
  NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
  CString temp;
  int number =pNMUpDown->iPos + pNMUpDown->iDelta;
  if ((number <= 0) || (number > m_exp_data->GetMaxWindows()))
  {
    *pResult = 1;
    return;
  }
  if ((number) != m_app_data->GetWindow())
  {
    *pResult = 1;
    temp.Format("%d", number);
    m_edit_window.SetWindowText(temp);
    m_scrollbar_vertical.SetScrollPos(number);
    m_controller.C_SetWindow(number);	
    m_converter.SetWindow( m_app_data->GetWindow() );
    ReDraw(1, 1);
  }
  else
  {
    *pResult = 0;
  }
}

// sets the root color when using the drop down box
void CRootflyDlg::OnSelchangeComboColor() 
{
  int color;
  
  if (m_selected_root_id != -1)
  {
    color = m_combo_color.GetCurSel();
    m_controller.C_SetRootColor(m_selected_root_id, color);
  }
  ReDraw(1, 0);
  EnableDisable();
}

// sets the root state when using the drop down box
void CRootflyDlg::OnSelchangeComboState() 
{
  int status;
  
  if (m_selected_root_id != -1)
  {
    status = m_combo_state.GetCurSel();
    if (status) m_controller.C_SetRootAlive(m_selected_root_id,true);
    else m_controller.C_SetRootAlive(m_selected_root_id,false);
  }
  ReDraw(1, 0);
  EnableDisable();
}

void CRootflyDlg::OnCHECKShowRootIDs() 
{
  //if (GetMode() == MODE_STARTINGMOVEALL){
  //  m_controller.C_SetShowRootID(false);
  //  return;
  //}
  
  if (m_checkbox_show_root_ids.GetCheck() == BST_CHECKED)
  {
    m_controller.C_SetShowRootID(true);
  }
  else
  {
    m_controller.C_SetShowRootID(false);
  }
  
  ReDraw(1, 0);
}

void CRootflyDlg::OnCHECKShowDeadRoots() 
{
  if (m_checkbox_show_dead_roots.GetCheck() == BST_CHECKED)
  {
    m_controller.C_SetShowDeadRoots(true);
  }
  else
  {
    m_controller.C_SetShowDeadRoots(false);
  }
  
  ReDraw(1, 0);
}

void CRootflyDlg::OnCHECKShowDiameters() 
{
  if (m_checkbox_show_diameters.GetCheck() == BST_CHECKED)
  {
    m_controller.C_SetShowDiameters(true);
  }
  else
  {
    m_controller.C_SetShowDiameters(false);
  }
  
  ReDraw(1, 0);
}

////////////////////// start condition code handling //////////////////////////

void CRootflyDlg::SetConditionDefault(const Root *root)
{
  CString default_code;
  int id = root->GetID();
  m_combo_custom.GetLBText(m_set_data->GetConditionDefaultIndex(), default_code);
  m_controller.C_SetConditionCode(id, default_code);
}

// sets the user code when using the drop down box
void CRootflyDlg::OnSelchangeComboCustom() 
{
  CString user_code;
  int index = m_combo_custom.GetCurSel();
  if (index >= 0)
  {
    m_combo_custom.GetLBText(index,user_code);
    m_controller.C_SetConditionCode(m_selected_root_id,user_code);
  }
}

// sets the condition code to the selected root
void CRootflyDlg::SetCondition(CString user_code)
{
  int index;
  index = m_combo_custom.FindStringExact(-1,user_code);
  if (index >= 0)
  {
    m_combo_custom.SetCurSel(index);
    m_controller.C_SetConditionCode(m_selected_root_id,user_code);
  } else if (user_code != "")
  {
    m_combo_custom.AddString(user_code);
    m_combo_custom.SelectString(-1,user_code);
    m_controller.C_SetConditionCode(m_selected_root_id,user_code);
    m_controller.C_AddConditionCode(user_code);
  }
}

// sets the user code when drop down is used
void CRootflyDlg::OnDropdownComboCustom() 
{
  CString user_code;  
  m_combo_custom.GetWindowText(user_code);
  SetCondition(user_code);  
  int index = m_combo_custom.GetCurSel();
  m_combo_custom.SetCurSel(index);
}

// selects the correct choice or adds if not found
// in the combocustom box
void CRootflyDlg::OnKillfocusComboCustom() 
{	  
  CString user_code;
  m_combo_custom.GetWindowText(user_code);
  OnSelchangeComboCustom();
  SetCondition(user_code);
}

// reloads the custom combo box with the set passed in
void CRootflyDlg::UpdateCustomCombo(std::vector<CString> set)
{
  m_combo_custom.ResetContent();
  for (int i = 0; i < set.size(); i++)
  {
    m_combo_custom.AddString(set.at(i));
  }
}

void CRootflyDlg::InitializeConditionSet()
{
  // sets up the initial custom box
  std::vector<CString> condition_set;
  condition_set.push_back("NO_CODE");
  m_controller.C_SetConditionDefaultIndex(0);
  m_controller.C_SetConditionSet(condition_set);
  m_combo_custom.AddString("NO_CODE");
}

// brings up dialog to add or remove condition codes
void CRootflyDlg::OnCondition() 
{
  ManageConditionCodesDlg dlg;
  std::vector<CString> codes;
  codes = m_set_data->GetConditionSet();
  if (codes.size() == 0)
  {
    InitializeConditionSet();
    codes = m_set_data->GetConditionSet();
  }
  dlg.SetConditionCodes(codes);
  dlg.SetDefaultIndex(m_set_data->GetConditionDefaultIndex());
  int ret = dlg.DoModal();
  if (ret == IDOK)
  {
    codes = dlg.GetConditionCodes();
    if (codes.size() == 0)
    {
      codes.push_back("NO_CODE");
      m_controller.C_SetConditionDefaultIndex(0);
    } 
    else 
    {
      m_controller.C_SetConditionDefaultIndex(dlg.GetDefaultIndex());
    }
    m_controller.C_SetConditionSet(codes);
    UpdateCustomCombo(codes);
    UpdateProperties( GetMode() == MODE_ROOTSELECTED ? m_selected_root_id : -1);
  }    
}

////////////////////// end condition code handling //////////////////////////

void CRootflyDlg::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
  int CurPos = m_scrollbar_vertical.GetScrollPos();
  int OldPos = CurPos;
  int min, max;
  m_scrollbar_vertical.GetScrollRange(&min,&max);
  
  // Determine the new position of scroll box.
  switch (nSBCode)
  {
  case SB_LEFT:      // Scroll to far left.
    CurPos = min;
    break;    
  case SB_RIGHT:      // Scroll to far right.
    CurPos = max;
    break;    
  case SB_ENDSCROLL:   // End scroll.
    return;
    break;    
  case SB_LINELEFT:      // Scroll left.
    if (CurPos > min)      CurPos--;     
    break;    
  case SB_LINERIGHT:   // Scroll right.
    if (CurPos < max)      CurPos++;
    break;    
  case SB_PAGELEFT:    // Scroll one page left.
    if (CurPos > min+((NUM_THUMBNAILS-1)/2))        CurPos -= ((NUM_THUMBNAILS-1)/2);  
    else CurPos = min;
    break;     
  case SB_PAGERIGHT:      // Scroll one page right
    if (CurPos <max-((NUM_THUMBNAILS-1)/2))        CurPos += ((NUM_THUMBNAILS-1)/2);  
    else CurPos = max;
    break;    
  case SB_THUMBPOSITION: // Scroll to absolute position. nPos is the position
    CurPos = nPos;      // of the scroll box at the end of the drag operation.
    break;    
  case SB_THUMBTRACK:   // Drag scroll box to specified position. nPos is the
    CurPos = nPos;     // position that the scroll box has been dragged to.
    break;
  }  
  // returns if the position didn't change
  if (OldPos == CurPos) return;
  if ( GetMode() != MODE_NOIMAGEDISPLAYED) SetMode(MODE_IMAGEDISPLAYED);
  
  SetMode(MODE_IMAGEDISPLAYED);
  m_selected_root_id = -1;
  m_scrollbar_vertical.SetScrollPos(CurPos);
  m_controller.C_SetWindow(CurPos);	
  m_converter.SetWindow( m_app_data->GetWindow() );
  CString temp;
  temp.Format("%d", CurPos);
  m_edit_window.SetWindowText(temp); 
  ReDraw(1, 1); 
}

void CRootflyDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{  
  int CurPos = m_scrollbar_horizontal.GetScrollPos();
  int OldPos = CurPos;
  int min, max;
  m_scrollbar_horizontal.GetScrollRange(&min,&max);
  
  // Determine the new position of scroll box.
  switch (nSBCode)
  {
  case SB_LEFT:                       // Scroll to far left.
    CurPos = min;
    break;    
  case SB_RIGHT:                      // Scroll to far right.
    CurPos = max;
    break;    
  case SB_ENDSCROLL:                  // End scroll.
    return;    
  case SB_LINELEFT:                   // Scroll left.
    if (CurPos > min)      CurPos--;     
    break;    
  case SB_LINERIGHT:                  // Scroll right.
    if (CurPos < max)      CurPos++;
    break;
    
  case SB_PAGELEFT:                   // Scroll one page left.
    if (CurPos > min+((NUM_THUMBNAILS-1)/2))        CurPos -= ((NUM_THUMBNAILS-1)/2);  
    else CurPos = min;
    break;    
  case SB_PAGERIGHT:                  // Scroll one page right
    if (CurPos <max-((NUM_THUMBNAILS-1)/2))        CurPos += ((NUM_THUMBNAILS-1)/2);  
    else CurPos = max;
    break;    
  case SB_THUMBPOSITION:       // Scroll to absolute position. nPos is the position
    CurPos = nPos;             // of the scroll box at the end of the drag operation.
    break;    
  case SB_THUMBTRACK:          // Drag scroll box to specified position. nPos is the
    CurPos = nPos;             // position that the scroll box has been dragged to.
    break;
  }  
  // returns if position didn't change
  if (OldPos == CurPos) return;
  
  // Set the new position of the thumb (scroll box).  
  if ( GetMode() != MODE_NOIMAGEDISPLAYED) SetMode(MODE_IMAGEDISPLAYED);
  SetMode(MODE_IMAGEDISPLAYED);
  m_selected_root_id = -1;
  m_scrollbar_horizontal.SetScrollPos(CurPos);
  m_controller.C_SetSession(CurPos);	
  CString temp;
  temp.Format("%d", CurPos);
  m_edit_session.SetWindowText(temp);
  ReDraw(1, 1);
  
}

void CRootflyDlg::ThumbnailClicked(int index, int type)
{  
  if ( GetMode() != MODE_NOIMAGEDISPLAYED ) SetMode(MODE_IMAGEDISPLAYED);
  m_selected_root_id = -1;
  int pic_clicked, max, prev_pos;
  CString new_location;
  if (type == 0)
  {
    max = m_exp_data->GetMaxWindows();
    pic_clicked = m_app_data->GetWindow();
    prev_pos = m_scrollbar_vertical.GetScrollPos()+index;
  } else 
  {
    max = m_exp_data->GetMaxSessions();
    pic_clicked = m_app_data->GetSession();
    prev_pos = m_scrollbar_horizontal.GetScrollPos()+index;
  }  
  if ((index < 0) && (pic_clicked < abs(index)+1)) return;
  else if ((index > 0) && (max-pic_clicked < index)) return;  
  new_location.Format("%d",pic_clicked+index);
  if (type == 0)
  {
    m_edit_window.SetWindowText(new_location);      
    m_scrollbar_vertical.SetScrollPos(prev_pos);
    m_controller.C_SetWindow(pic_clicked+index);
    m_converter.SetWindow( m_app_data->GetWindow() );
  } else 
  {
    m_edit_session.SetWindowText(new_location);      
    m_scrollbar_horizontal.SetScrollPos(prev_pos);
    m_controller.C_SetSession(pic_clicked+index);
  }
  ReDraw(1,1);
}

bool CRootflyDlg::CheckForThumbnailClick(CPoint point)
{
  if ( GetMode() == MODE_NOFILELOADED ) return false;   
  CRect rect;
  bool clicked = false;
  for (int i = 0; i < 2; i++)
  {
    for (int index = -1*(NUM_THUMBNAILS-1)/2 ; index<(NUM_THUMBNAILS+1)/2 ; index++)
    {
      rect = GetThumbnailRect(index,i);
      if (rect.PtInRect(point))
      {
        ThumbnailClicked(index,i);
        clicked = true;
      }
    }
  }
  return clicked;
}

void CRootflyDlg::OnEditSessionDates()
{
  if (!m_image_file_format.DateIsIncluded())
  {
    SessionDatesDlg dlg;
    std::vector<Date> dates;
    dates = m_exp_data->GetSessionDates();
    dlg.SetSessionDates(dates);
    int ret = dlg.DoModal();
    if (ret == IDOK)
    {
      dates = dlg.GetSessionDates();
      m_controller.C_SetSessionDates(dates);          
    }
  }
}

/////////////////////////////////////////////////////////////////////////////////////////
//                                                                                     //
//                                 DRAWING FUNCTIONS                                   //
//                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////

/**
@ brief: ReDraw updates only the necessary images
@ param: main_changed - if 1 update main image, if 0 do not update main image
@ param: update_thumbnails - if 1 redraw thumbnails, if 0 do not redraw thumbnails
*/

void CRootflyDlg::ReDraw(int main_changed, int update_thumbnails)
{   
  CClientDC dc(this);  
  CRect section_rect, clip_rect;
  m_images_section.GetWindowRect(&section_rect);
  
  section_rect.DeflateRect(1,7,2,2);
  clip_rect = section_rect;
  ScreenToClient(clip_rect);
  
  CRgn clip_rgn;
  clip_rgn.CreateRectRgnIndirect(&clip_rect);
  CMemDC mdc(&dc);
  
  if (update_thumbnails) UpdateThumbnails();   
  if (main_changed)
  {
    UpdateMainImage(mdc); 
    // run automatic detector if it hasn't been run.
    if(m_set_data->GetAutoDetectOnLoad() && GetMode() == MODE_IMAGEDISPLAYED) RunAutomaticRootDetector();
  } 

  UpdateProperties(m_selected_root_id); //this should be sufficient to ensure the root exists, i.e. root can not be selected if it does not exist
  
  // shows the mm conversion on the window
  CString str;
  str.Format("%5.1f pixels per mm", m_exp_data->GetMMConversion());

  Status_bar.SetPaneText(1,str);  //STATUSBAR
  //m_mm_text.SetWindowText(str);
  
  m_graphics.SetClipRegion(dc, IsZoomOutMode());

  // placeholder for new clipping region on zoom in
  //m_graphics.SetClipRegion(dc, clip_rect);

  m_graphics.DrawAllRoots(mdc, GetMode(), IsZoomOutMode());      // draw all the existing roots in the data structure
  m_graphics.DrawCurrentRoot(mdc, GetMode(), m_current_root_being_drawn, m_last_mouse_point);   // draw the temporary new root that is being created
  if ( m_selected_root_id > 0)
  {
    // draw the selected root (or the root under the mouse, i.e., the one that will be selected if the mouse button is pressed)
    m_graphics.DrawSelectedRoot( mdc, m_selected_root_id, GetMode(), m_last_mouse_point, m_extending_from_tail );  
  }
}

// checks to see if the root detector has been run on the current image
// will run if it has not been run, will do nothing if it hasn't been run
void CRootflyDlg::RunAutomaticRootDetector()
{
  if (GetMode() == MODE_NOIMAGEDISPLAYED || GetMode() == MODE_NOFILELOADED)  return;
  const Tube *tube = m_exp_data->GetTube( m_app_data->GetTube() );  // current tube
  int window = m_app_data->GetWindow();   // current window
  int session = m_app_data->GetSession(); // current session
  
  // check whether image has been scanned (i.e., whether automatic root detector has been run)
  //if (!tube || !tube->GetImageScanned(window,session))
  //{

    // if any roots exist in the current tube, then do not run detector
    //need to check window and not tube!
    if (tube && tube->AutoDetectGetRootsInWindow(window) > 0)  return;
  
      // run the automatic root detector here  
      blepo::ImgBgr img;    	
      img  = m_app_data->GetWindowImage(2);
      unsigned char *img_data = img.BytePtr();
      std::vector<DetectedRoot> detected_roots;

      detected_roots.resize(30);
      for(int k = 0; k < detected_roots.size(); k++)
      {
        detected_roots.at(k).points.resize(100);
        detected_roots.at(k).extremity.resize(50);
      }
  
      //the actual autodetect function
      MyDetectRoots(img.Width(), img.Height(), img_data, &detected_roots);
  
  
      if(GetMode() == MODE_NOIMAGEDISPLAYED) return;
      m_selected_root_id = -1;
      m_current_root_being_drawn.ClearAll();
  
      //mode: DRAWINGNEWROOT
      RPoint *rpt;
      RPoint *rpt1;
      RDiameter *rdm;
      RDiameter *rd;
      RDiameter *rd2;
      Root *root;
      RootInstance *inst;
      const std::vector<RPoint> *line;  
      vector<RPoint>::const_iterator iter;

      for(int i = 0; i < detected_roots.size(); i++) 
      {
        if (m_set_data->GetAutoDetectDiameterCheck() && ((detected_roots.at(i).diameter*2) > m_set_data->GetAutoDetectDiameterValue())) continue;
        if (detected_roots.at(i).diameter < 0) continue;
		// Added length code
	/*	// updates the length of the root
        double templength = 0;
        double dx, dy;
        const std::vector<RPoint> *points= inst->GetAllPoints();
        for (int j=1; j<points->size(); j++)
		{
           dx = (points->at(j).x-points->at(j-1).x) / m_exp_data->GetMMConversion();
           dy = (points->at(j).y-points->at(j-1).y) / m_exp_data->GetMMConversion();
           templength += sqrt(pow(dx,2)+pow(dy,2));
		}
        tempstr.Format("%.2f mm",templength);
        // m_edit_length.SetWindowText(tempstr);
        // Added length code */
        for(int j = 0; j < detected_roots.at(i).points.size(); j++) {
          if(detected_roots.at(i).points.at(j).x < 0 || detected_roots.at(i).points.at(j).y < 0) continue;
          rpt1 = m_current_root_being_drawn.CreatePoint();
          assert(rpt1); if(!rpt1) return;
          rpt1->x = detected_roots.at(i).points.at(j).x;
          rpt1->y = detected_roots.at(i).points.at(j).y;
          rpt1->window = window;
        }
    
    
        rdm = m_current_root_being_drawn.CreateDiameter();
        rdm->r_point.x = detected_roots.at(i).points.at(0).x;
        rdm->r_point.y = detected_roots.at(i).points.at(0).y;
        rdm->radius = detected_roots.at(i).diameter;
        rdm->r_point.window = window;
    
        //current mode: LOCATINGDIAMETER
        //current mode: SIZINGDIAMETER
        //root is now created
        rd = m_current_root_being_drawn.GetDiameter(0);
        tube = m_exp_data->GetTube( m_app_data->GetTube() );
        if (!tube) tube = m_controller.C_CreateTube(m_app_data->GetTube());
        line = m_current_root_being_drawn.GetAllPoints();    
        if (line->size() == 0)  return;
        root = m_controller.C_CreateRoot( line->at(0).window );    
        inst = m_controller.C_CreateRootInstance( root->GetID() );  
        for ( iter = line->begin(); iter != line->end(); iter++ )
        {
          rpt = inst->CreatePoint();
          rpt->x = iter->x;
          rpt->y = iter->y;
          rpt->window = iter->window;     
        }
        rd2 = m_controller.C_CreateDiameter( root->GetID() );
        rd2->r_point.x = rd->r_point.x;
        rd2->r_point.y = rd->r_point.y;
        rd2->r_point.window = rd->r_point.window;
        rd2->radius = rd->radius;
        // sets the default condition code for the root
        SetConditionDefault(root);
        m_selected_root_id = -1;
        m_current_root_being_drawn.ClearAll();
        ReDraw(1,0);
      }
  
      //free memory reserved for roots
      while (detected_roots.size() > 0)
      {
          while (detected_roots.at(0).points.size() > 0)  detected_roots.at(0).points.erase( detected_roots.at(0).points.begin() );
          while (detected_roots.at(0).extremity.size() > 0)  detected_roots.at(0).extremity.erase( detected_roots.at(0).extremity.begin() );
          detected_roots.erase( detected_roots.begin() );
      }
  
      //m_controller.C_SetImageScanned(true);
      //}
      SetMode(MODE_IMAGEDISPLAYED);
    //}
  //}
}

/////////////////////////////////////////////////////////////////////////////////////////
//                                                                                     //
//                              MOUSE BUTTON FUNCTIONS                                 //
//                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////

void CRootflyDlg::OnLButtonDown(UINT nFlags, CPoint point)
{  
//  {
//    CString foo;
//    RPoint p = m_converter.ClientToImage( point );
//    foo.Format("(%d,%d;%d)", p.x, p.y, p.window);
//    AfxMessageBox(foo);
////    return;
//  }
  CheckForThumbnailClick(point);   
      
  CRect main = IsZoomOutMode() 
    ? m_converter.GetExtendedClientRect(m_app_data->GetWindow()-1, m_app_data->GetWindow()+1)
    : m_converter.GetClientRect( m_app_data->GetWindow() );
  if (main.PtInRect( point ))
  {
    switch( GetMode() )  
    {
    case MODE_STARTINGMOVEALL:
      LeftClickWhileStartingMoveAll( m_converter.ClientToImage( point ) );      
      break;

    case MODE_MOVINGSINGLEROOT:
      LeftClickWhileMovingSingleRoot( m_converter.ClientToImage( point ) );
      break;

    case MODE_MOVINGALLROOTS:
      LeftClickWhileMovingAllRoots( m_converter.ClientToImage( point ) );    
      break;

    case MODE_SIZINGDIAMETER:
      LeftClickWhileSizingDiameter( m_converter.ClientToImage( point ) );
      break; 

    case MODE_DRAWINGNEWROOT:       
      LeftClickWhileDrawingNewRoot( m_converter.ClientToImage( point ) ); 
      break;     

    case MODE_ROOTSELECTED:    
      LeftClickWhileRootSelected( m_converter.ClientToImage( point ) );
      break;
    
    case MODE_NOIMAGEDISPLAYED:
      break;

    case MODE_EXTENDINGENDPOINT:
      LeftClickWhileExtendingEndpoint( m_converter.ClientToImage( point ) );
      break;

    case MODE_IMAGEDISPLAYED: 
      if (m_over_root)
      {
        SetMode(MODE_ROOTSELECTED);            
        ReDraw(1, 0);
      }   
      break; 
    
    case MODE_RESIZINGDIAMETER:
      SetMode(MODE_ROOTSELECTED);
      ReDraw(1,0);
      break;

    case MODE_MOVINGRPOINT:
      SetMode(MODE_ROOTSELECTED);          
      ReDraw(1, 0);
      break;

    case MODE_LOCATINGDIAMETER: 
      SetMode(MODE_SIZINGDIAMETER);       
      ReDraw(1, 0);
      break;     
    
    case MODE_REPOSITIONDIAMETER:
      SetMode(MODE_ROOTSELECTED);          
      ReDraw(1, 0);
      break;
    }   
  }
}

void CRootflyDlg::OnLButtonUp(UINT nFlags, CPoint point)
{
  
}

void CRootflyDlg::OnLButtonDblClk(UINT nFlags, CPoint point)
{  
  std::vector<RPoint>::const_iterator r_iter;
  RDiameter *rd2;
  RPoint *rpt;  
  RootInstance *r_inst;
  const Root *root;  
  const Tube *tube;
  const std::vector<RPoint> *line;  
  
  RPoint conv_point, temp;
  CRect main;
  RDiameter *rdm;

  int window = m_app_data->GetWindow();
  main = IsZoomOutMode() 
    ? m_converter.GetExtendedClientRect(window-1, window+1)
    : m_converter.GetClientRect( window );
  conv_point = m_converter.ClientToImage( point );
  if (main.PtInRect( point ))
  {
    switch( GetMode() )
    {
    case MODE_DRAWINGNEWROOT: 
      while ( m_current_root_being_drawn.GetAllPoints()->size() < 2 ) 
        LeftClickWhileDrawingNewRoot( m_converter.ClientToImage( point ) );
      rdm = m_current_root_being_drawn.CreateDiameter();
      rdm->r_point.x = conv_point.x;
      rdm->r_point.y = conv_point.y;
      rdm->radius = 8;
      rdm->r_point.window = m_app_data->GetWindow();
      SetMode(MODE_LOCATINGDIAMETER);            
      break;    
    case MODE_LOCATINGDIAMETER:
    case MODE_SIZINGDIAMETER:
      rdm = m_current_root_being_drawn.GetDiameter(0);
      temp = FindNearestPointOnRoot( conv_point, m_current_root_being_drawn );
      rdm->r_point.x = temp.x;
      rdm->r_point.y = temp.y;
      rdm->r_point.window = temp.window;
      rdm->radius = 8;
//      rdm->r_point.window = window; 
//      rdm->radius = 8; 
      
      tube = m_exp_data->GetTube( m_app_data->GetTube() );
      if ( tube == NULL ) tube = m_controller.C_CreateTube(m_app_data->GetTube());
      line = m_current_root_being_drawn.GetAllPoints();    
      assert(line->size() > 0);
      if (line->size() == 0)  break;
      root = m_controller.C_CreateRoot( line->at(0).window );    
      r_inst = m_controller.C_CreateRootInstance( root->GetID() );
      assert( r_inst );    
      for ( r_iter = line->begin(); r_iter != line->end(); r_iter++ )
      {
        rpt = r_inst->CreatePoint();
        rpt->x = r_iter->x;
        rpt->y = r_iter->y;
        rpt->window = r_iter->window;     
      }
      rd2 = m_controller.C_CreateDiameter( root->GetID() );
      rd2->r_point.x = rdm->r_point.x;
      rd2->r_point.y = rdm->r_point.y;
      rd2->r_point.window = rdm->r_point.window;
      rd2->radius = rdm->radius;
      
      // sets the default condition code for the root
      SetConditionDefault(root);
      
      SetMode(MODE_IMAGEDISPLAYED);
      m_current_root_being_drawn.ClearAll();
      ReDraw(1, 0);
      break;                  
    default:
      break;        
    }
  }      
}

void CRootflyDlg::OnRButtonDown(UINT nFlags, CPoint point)
{  
  if ( GetMode() == MODE_EXTENDINGENDPOINT )
  {
    // cancel extending endpoint
    SetMode(MODE_ROOTSELECTED);
    m_extending_from_tail = false;
    m_extending_from_head = false;
    m_current_root_being_drawn.ClearAll();
    ReDraw(1,0);
  }  
  else if ( GetMode() == MODE_ROOTSELECTED )  
  {
     if ( m_over_root )
     {
       // show context menu
       CMenu menu;
       menu.LoadMenu(IDR_MENU_CONTEXT);
       CMenu* popup;
       if( m_selected_index > -1 )  //Delete Button if Point clicked
       { 
         popup = menu.GetSubMenu(0);  
       }
       else   //Add Point Button if other part of root clicked
       {  
         const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
         assert(tube);  if (!tube)  return;
         const Root *root = tube->GetRoot(m_selected_root_id);
         assert(root);  if (!root)  return;
         const RootInstance *inst = root->GetRootInstance(m_app_data->GetSession());
         assert(inst);  if (!inst)  return;
         popup = menu.GetSubMenu(1);  
         temporary_point_to_add = m_converter.ClientToImage(point);
         FindNearestPointOnRootSeg(temporary_point_to_add, *inst, &section_start_point_index);  //Get segment to add point on
       }
       assert(popup != NULL);
       CPoint pt = point;
       ClientToScreen(&pt);
       popup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, this);
       SetMode(MODE_IMAGEDISPLAYED); //keeps it in the right mode after closing the menu
     }
  }  
}

void CRootflyDlg::OnMouseMove(UINT nFlags, CPoint point)
{  
  m_last_mouse_point = point;   
    
  CRect main = IsZoomOutMode() 
    ? m_converter.GetExtendedClientRect(m_app_data->GetWindow()-1, m_app_data->GetWindow()+1)
    : m_converter.GetClientRect( m_app_data->GetWindow() );
  if (main.PtInRect( point ))
  {
    switch( GetMode() )
    {        
    case MODE_MOVINGSINGLEROOT:
      MoveMouseWhileMovingSingleRoot( m_converter.ClientToImage( point) );
	  break;

    case MODE_MOVINGALLROOTS:
      MoveMouseWhileMovingAllRoots( m_converter.ClientToImage( point) );
      break;

    case MODE_SIZINGDIAMETER: 
      MoveMouseWhileSizingDiameter( m_converter.ClientToImage( point) );
      break;
      
    case MODE_RESIZINGDIAMETER:      
      MoveMouseWhileResizingRDiameter( m_converter.ClientToImage( point) );
      break;
      
    case MODE_LOCATINGDIAMETER:       
      MoveMouseWhileLocatingDiameter( m_converter.ClientToImage( point) );
      break;  
      
    case MODE_ROOTSELECTED:      
        MoveMouseWhileRootSelected( m_converter.ClientToImage( point) );
        break;
      
    case MODE_REPOSITIONDIAMETER:      
      MoveMouseWhileMovingRDiameter( m_converter.ClientToImage( point) );
      break;
      
    case MODE_MOVINGRPOINT:      
      MoveMouseWhileMovingRPoint( m_converter.ClientToImage( point) );
      break;
      
    case MODE_EXTENDINGENDPOINT:
      MoveMouseWhileExtendingEndpoint( m_converter.ClientToImage( point) );
      break;

    case MODE_IMAGEDISPLAYED:      
        MoveMouseWhileImageDisplayed( m_converter.ClientToImage( point) );
        break;

    case MODE_DRAWINGNEWROOT:    
      SetCursor(m_cur_pen);
      ReDraw( 1, 0 );
      break;
      
    case MODE_NOIMAGEDISPLAYED:
      break;

    case MODE_NOFILELOADED:
      break;

    case MODE_STARTINGMOVEALL:
      break;

    default:  
      assert(0);  // should always be in a mode
      break;
    }
  }
} 

/////////////////////////////////////////////////////////////////////////////////////////
//                                                                                     //
//                            FILE RELATED FUNCTIONS                                   //
//                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////

/**

  Action: Whenever user selects: File->Exit  
  Should close main window         
  */
  void CRootflyDlg::OnFileExit() 
  {
    // Send message to close window
    PostMessage(WM_CLOSE);
  }
  
  // Scan directory for max/min window/session, and set ranges of UI accordingly.
  void CRootflyDlg::SetNavigation(const CString& file_directory)
  {
    // scan all files and set max/min window/session
    int max_window = 0, min_window = 10000, max_session = 0, min_session = 10000;    
    m_image_file_format.FindNavigationLimits(file_directory,&max_window,&min_window,&max_session,&min_session, this);
    m_controller.C_SetMaxWindows(max_window);
    m_controller.C_SetMaxSessions(max_session);
    
    // set values of Navigation pane
    m_app_data = m_rfy_data->GetApplicationData();
    // displays tube without padded zeros
    m_edit_tube.SetWindowText(StripPaddedZeros(m_app_data->GetTube()));
    m_spin_window.SetPos(m_app_data->GetWindow());
    m_spin_session.SetPos(m_app_data->GetSession());
    
    // sets the ranges of Navigation pane
    //m_exp_data = m_rfy_data->GetExperiment();
    m_spin_window.SetRange(1,m_exp_data->GetMaxWindows());
    m_spin_session.SetRange(1,m_exp_data->GetMaxSessions());
    m_scrollbar_vertical.SetScrollRange(1,m_exp_data->GetMaxWindows());
    m_scrollbar_horizontal.SetScrollRange(1,m_exp_data->GetMaxSessions());
    
    // set scroll bars 
    m_scrollbar_vertical.SetScrollPos(m_app_data->GetWindow());
    m_scrollbar_horizontal.SetScrollPos(m_app_data->GetSession());
  }
  
  // Scan directory for valid files.  If none found, reprompt user for valid image directory
  // 'directory' is both input and output
  bool CRootflyDlg::ValidateDirectory(CString* directory)
  {
    CString& file_directory = *directory;
    while(!m_image_file_format.ImageFileExistsInDirectory(file_directory))
    {
      UINT	uFlags	  = BIF_RETURNONLYFSDIRS | BIF_USENEWUI;
      CFolderDialog dlg(_T("No Valid Images Found. Choose Folder where Images are Located" ), file_directory, this, uFlags);
      if (dlg.DoModal() == IDOK)
      {
        file_directory = dlg.GetFolderPath();
        file_directory += "\\";
        m_controller.C_SetImageDir(file_directory);
        m_controller.C_SetModify(true);

//        filename = file_directory + m_app_data->GetFileName();
      }
      else return false;
    }   
//    m_file_directory = file_directory;
    return true;
  }
  
  /**
  /  
  /    @Action: Whenever user selects: File->New
  /             Creates a Browse Folder Dialog and 
  /             allows user to select location of 
  /             new experiment.
  /
  */
  void CRootflyDlg::OnFileNew() 
  {
    UINT	uFlags	  = BIF_RETURNONLYFSDIRS | BIF_USENEWUI;
    
    m_first_save = false;
    
    // Let user choose folder that contains images for the new project
    CString file_directory;
    
    // Customize string in file dialog
    CString str = "Choose Folder where Images are Located";
    bool valid_directory = false;
    while (!valid_directory)
    {
      // gets the default directory
      file_directory = m_set_data->GetFileDir();
      
      CFolderDialog dlg(str/*_T("Choose Folder where Images are Located" )*/, file_directory, this, uFlags);
      int fdlgid = dlg.DoModal();
      if (fdlgid == IDOK)
      {          
        // Look for images in chosen folder
        CFileFind ff;
        CString tmp = dlg.GetFolderPath();
        // sets the new default directory
        m_controller.C_SetFileDirectory(tmp);
        
        if (ff.FindFile(tmp+"\\*.jpg") || ff.FindFile(tmp+"\\*.bmp") || ff.FindFile(tmp+"\\*.jpeg"))
        {          
          // Let user select the image file name format
          OptionsDlg optionsdlg(&m_image_file_format,this);
          int id = optionsdlg.DoModal();
          if (id == IDOK)
          {
            CString image_directory = dlg.GetFolderPath();
            image_directory += "\\";
            
            //if (ValidateDirectory(&image_directory) == false) return;
            if (!m_image_file_format.ImageFileExistsInDirectory(image_directory))
            {
              // do nothing have to find valid file first
              valid_directory = false;
            } else
            {
              // valid file found. start new project
              valid_directory = true;
              
              if ( m_exp_data ) m_controller.C_ClearAllExperiment();
              m_controller.C_SetImageFileFormat(m_image_file_format);
              m_controller.C_SetImageDir(image_directory);
              
              CString file_name = image_directory + DEFAULT_FILENAME;
              m_controller.C_CreateNewProject(_T(file_name),_T(image_directory));
              SetNavigation(image_directory);
              if (!m_image_file_format.DateIsIncluded())
              {
                SessionDatesDlg dlg;
                std::vector<Date> dates;
                dlg.SetMaxSessions(m_exp_data->GetMaxSessions());
                dlg.DoModal();
                dates = dlg.GetSessionDates();
                m_controller.C_SetSessionDates(dates);
              } else
              {
                // dates included in file, get dates from file name to store
                ImageInfo temp_info;
                std::vector<Date> dates;
                dates = m_image_file_format.GetDatesFromFile(image_directory,m_exp_data->GetMaxSessions(),this);                
                m_controller.C_SetSessionDates(dates);
              }
              
              // get main image file name
              file_name = m_image_file_format.GetFileName(image_directory,m_app_data->GetTube(),m_app_data->GetWindow(),m_app_data->GetSession());
              
              SetMode(MODE_NOIMAGEDISPLAYED);              
              
              m_controller.C_SetCurrentImage(file_name);
              m_controller.C_SetModify(false);  // unnecessary, but just in case

              ClearSelection();
              ReDraw(1,1);
            }
          } // OptionsDlg OK
          else return; // OptionsDlg Cancel
        } // If images in directory
        else str = "No Images Found. Choose Folder where Images are Located"; //AfxMessageBox("No image files found in directory.");
      } // ChooseImageDirectoryDlg OK
      else return; // ChooseImageDirectory Cancel
    } // while not valid directory
  } 
  
  /**  
  /    @Action: Whenever user selects: File->Open
  /             Creates an open file dialog
  /     
  */
  void CRootflyDlg::OnFileOpen() 
  {
    //check to see if any modifications have been made to the file
    if (m_app_data->GetModify())
    {
      int return_value = AfxMessageBox("Project data has changed. Save changes?", MB_YESNOCANCEL);
      if (return_value == IDCANCEL)  return;
      else if (return_value == IDYES)
      {
        OnFileSave();  //call the save function if things have changed
      }
    }
    
    CFileDialog fdlg(TRUE,NULL,NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,"Rootfly File(*.rfy)|*.rfy|All Files(*.*)|*.*||",this);
    
    if (fdlg.DoModal() == IDOK)
    {
      
      CString filename = fdlg.GetPathName();
      
      // Check for valid .rfy extension
      if (filename.Right(3) != "rfy")
      {
        filename += ".rfy";
        //AfxMessageBox("Error: File should be of extension RFY");
        //return;
      }
      
      // Load file
      int status = m_controller.C_LoadFile(filename);
      if (status != ERRORS::RFY_LOADERROR_SUCCESS)
      {
        if (status == ERRORS::RFY_LOADERROR_VERSION)
          AfxMessageBox("Error! Invalid file version");
        else if (status == ERRORS::RFY_LOADERROR_FILECORRUPT)
          AfxMessageBox("Error! Loading corrupt file");
        else if (status == ERRORS::RFY_LOADERROR_NOFILEFOUND)
          AfxMessageBox("Error! Could not load file");
        else if (status == ERRORS::RFY_LOADERROR_USERCANCELED)
        {}
        SetMode(MODE_NOFILELOADED);
        return;
      }    
      
      SetMode(MODE_IMAGEDISPLAYED);
      
      m_image_file_format = m_controller.C_GetImageFileFormat();
      
      CString file_directory = m_exp_data->GetImageDir();
      
      // while no valid images were found, keeps asking user to specify new directory
      if (ValidateDirectory(&file_directory) == false) return;
      
      // Set initial tube, window, and session info
      ImageInfo ii = m_image_file_format.GetImageInfoOfFirstFile(file_directory);
      m_controller.C_SetExperiment(ii.expt);
      m_controller.C_SetTube(ii.tube);
      m_controller.C_SetWindow(ii.window);
      m_converter.SetWindow( m_app_data->GetWindow() );
      m_controller.C_SetSession(ii.session);
      
      // If the current experiment is a converted old file, then
      // convert to the new format (i.e., image coordinates instead of client coordinates)
      if (m_exp_data->IsOldFile())
      {      
        CWaitCursor wait;
        m_controller.C_ConvertOldRoots(file_directory, m_image_file_format, this);
      }      
      
      // Get the min/max window/sesion
      SetNavigation(file_directory);
      
      // make sure to have dates from file
      bool included = false;
      vector<Date> temp_dates = m_exp_data->GetSessionDates();
      for (int i = 0; i < temp_dates.size(); i++)
      {
        if (temp_dates.at(i).month != -1) included = true;
      }
      if (!included && m_image_file_format.DateIsIncluded())
      {
        
        temp_dates = m_image_file_format.GetDatesFromFile(m_exp_data->GetImageDir(), m_exp_data->GetMaxSessions(),this);
        m_controller.C_SetSessionDates(temp_dates);
      }
      
      // get main image file name
      filename = m_image_file_format.GetFileName(file_directory,m_app_data->GetTube(),m_app_data->GetWindow(),m_app_data->GetSession());
      
      m_first_save = true;
      
      ///////////////////////////////////////////////
      // loads custom codes from file and adds them to the list
      const vector<Tube> *t1;   
      const vector<Root> *r1;
      const vector<RootInstance> *ri;
      CString temp;
      
      t1 = m_exp_data->GetAllTubes();
      if (!t1) return;
      for (i = 0; i < t1->size(); i++)
      {
        r1 = t1->at(0).GetAllRoots();
        if (!r1) return;
        for (int j = 0; j < r1->size(); j++)
        {
          ri = r1->at(j).GetAllRootInstances();
          if (!ri) return;
          for (int k = 0; k < ri->size(); k++)
          {
            
            SetCondition(ri->at(k).GetConditionCode());
          }
        }
      }
      ///////////////////////////////////////////////
      
      SetMode(MODE_IMAGEDISPLAYED);
      m_controller.C_SetCurrentImage(filename); 
      ClearSelection();
      ReDraw(1, 1);
    } 
  }  
  
  /**
  /  
  /    @Action: Whenever user selects: File->SaveAs
  /             Creates new Save As dialog
  /     
  */
void CRootflyDlg::OnFileSaveAs() 
{
  g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 1\n");
  //Create new save as dialog that only allows *.rfy files to be saved
  CFileDialog fdlg(FALSE,"rfy",NULL,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,"Rootfly Files(*.rfy)|*.rfy|All Files(*.*)|*.*||",this);
  g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 2\n");
  
  if (fdlg.DoModal() == IDOK)
  {
    g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 3\n");
    m_first_save = true;
    g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 4\n");
    CString filename = fdlg.GetPathName();
    g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 5\n");
    m_controller.C_SetFilename(filename);
    g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 6\n");
    m_controller.C_SaveFile(filename);      
    g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 7\n");
  } 
  g_debug_logfile.Print("CRootflyDlg::OnFileSaveAs() 8\n");
}

/**
/  
/    @Action: Whenever user selects: File->Save    
/             Sends request for a save action.      
/        
*/
void CRootflyDlg::OnFileSave() 
{
  bool write_to_debug_log = true;
  if (write_to_debug_log) g_debug_logfile.Open("rootfly_logfile.txt");
  g_debug_logfile.Print("CRootflyDlg::OnFileSave() 1\n");
  CString filename = m_app_data->GetFileName();
  
  g_debug_logfile.Print("CRootflyDlg::OnFileSave() 2\n");
  if (filename == "")    AfxMessageBox("Filename not specified");
  else if (!m_first_save)  OnFileSaveAs();
  else
  {
    g_debug_logfile.Print("CRootflyDlg::OnFileSave() 3\n");
    m_first_save = true;
    g_debug_logfile.Print("CRootflyDlg::OnFileSave() 4\n");
    m_controller.C_SaveFile(filename);
    g_debug_logfile.Print("CRootflyDlg::OnFileSave() 5\n");
  }
  g_debug_logfile.Print("CRootflyDlg::OnFileSave() 6\n");
  g_debug_logfile.Close();
}

void CRootflyDlg::OnDebugShowdebuglogfilelocation() 
{
  char a[1000];
  _getcwd(a, 1000);
  CString s = "Log file should be \r\n";
  s += a;
  s += "\\rootfly_logfile.txt";
  AfxMessageBox(s);
}

/**
* This is the export function for the entire program - NOTE: The condition codes are enabled if DISPLAY_CONDITION_CODE is defined below.
*/

void CRootflyDlg::OnFileExport() 
{
  
  //This controls whether or not the export function will include a condition code for each root session.
  //Comment it out to disable the condition code.
  CFileDialog fdlg(FALSE,"csv",NULL,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,"CSV(*.csv)|*.csv|All Files(*.*)|*.*||",this);
  fdlg.m_ofn.lpstrTitle = "Export";
  if (fdlg.DoModal() == IDOK)
  {
    
    CString filename = fdlg.GetPathName();  
    
    //      bool success = m_exp_date->Export(filename);
    
    //Make sure the file can be opened/written to
    
    m_controller.C_Export(filename);
  }
}
/////////////////////////////////////////////////////////////////////////////////////////
//                                                                                     //
//                               HELPER FUNCTIONS                                      //
//                                                                                     //
/////////////////////////////////////////////////////////////////////////////////////////

// This function searches through every root in the current window
// and
void CRootflyDlg::FindNearestPoint( int x, int y, RDiameter *temp ) const
{
  const int threshold = 18;  // how close point has to be to hit
  pt2d a,b,p,c;  
  double dist, min_dist = 999999.9;   
  std::vector<Root>::const_iterator root_ptr;
  const RootInstance *ri;
  std::vector<RPoint>::const_iterator p1;
  std::vector<RPoint>::const_iterator p2;
  temp->radius = -1;
  
  int window = m_app_data->GetWindow();
  
  const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
  if (tube == NULL)   return;
  const std::vector<Root> *root = tube->GetAllRoots();
  if (root == NULL) return;
  for ( root_ptr = root->begin(); root_ptr != root->end(); root_ptr++ )
  {
    if (root_ptr->GetPrimaryWindow() == window)
    {
      ri = root_ptr->GetRootInstance(m_app_data->GetSession());
      if (!ri)  continue;  // no root instance, try next root
      p1 = &(ri->GetAllPoints()->at(0));
      p2 = &(ri->GetAllPoints()->at(1));   
      while( p2 != ri->GetAllPoints()->end())
      {     
        a.x = p1->x;
        a.y = p1->y;
        b.x = p2->x;
        b.y = p2->y;
        p.x = x;
        p.y = y;
        dist = DistPt2Seg(a,b,p,&c);   
        if ( dist < min_dist )
        {
          min_dist = dist;
          temp->r_point.x = (int)(c.x);
          temp->r_point.y = (int)(c.y);
          if ( min_dist < threshold ) 
          {
            temp->radius = min_dist;
            temp->r_point.window = root_ptr->GetID();  // window contains ID !!!  
          }
        } 
        p1++;
        p2++;          
      }
    }
  } 
  return;
}

// This function returns the root ID of the smallest root diameter that encircles (x,y).  
// Returns -1 if (x,y) is outside all the diameters.
int CRootflyDlg::FindNearestDiameter( int x, int y) const
{ 
  int selected = -1;
  if ( m_set_data->ShowDiametersIsEnabled() )
  {
    const int threshold = 4;  // how close point has to be to hit
    double min_radius = 9999999.9;   
    std::vector<Root>::const_iterator root_ptr;
    const RootInstance *ri;
    int window = m_app_data->GetWindow();   
    
    const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());  
    if (tube == NULL)  return selected;
    const std::vector<Root> *root = tube->GetAllRoots();   

    ////////////// placed in by DJ to see if it fixes selecting the diameter behind the currently selected diameter
 /*   const Root *selected_root = tube->GetRoot(m_selected_root_id);
    if(selected_root != NULL)
    {
      const RootInstance *selected_ri = selected_root->GetRootInstance(m_app_data->GetSession());
      if(selected_ri != NULL)
      {
        if(IsPtInsideDiameter(x,y,selected_ri->GetDiameter(0),threshold))
          return selected;
      }
    }
 */   ////////////// end of fix
    if (root == NULL) return selected;
    for ( root_ptr = root->begin(); root_ptr != root->end(); root_ptr++ )
    {
      if (root_ptr->GetPrimaryWindow() ==  window )
      {
        ri = root_ptr->GetRootInstance(m_app_data->GetSession());
        if (ri == NULL)      continue;    
        const RDiameter* rdia = ri->GetDiameter(0);
        bool inside = IsPtInsideDiameter(x, y, rdia, threshold);
        if ( inside && rdia->radius < min_radius ) 
        { 
          min_radius = rdia->radius;
          selected = root_ptr->GetID();
        }  
      }
    }
  }
  return selected;
}

// This function will updates the starting point of the root segment the diameter is located in
// and the ratio of the distance from that point to the diameter over the total distance of that segment
// the data is stored in the variable section_start_point_index and diameter_position_ratio    
void CRootflyDlg::UpdateDiameterPositionRatio()
{
    const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
    assert(tube);  if (!tube)  return;
    const Root *root = tube->GetRoot(m_selected_root_id);
    assert(root);  if (!root)  return;
    const RootInstance *inst = root->GetRootInstance(m_app_data->GetSession());
    assert(inst);  if (!inst)  return;
    const RDiameter *diam = inst->GetDiameter(0);
    assert(diam);  if (!diam)  return;

    //get first point of section of root with diameter
    FindNearestPointOnRootSeg( diam->r_point, *inst, &section_start_point_index);
    //get points of section
    const RPoint* start_point = inst->GetPoint(section_start_point_index);
    const RPoint* other_point = inst->GetPoint(section_start_point_index + 1);
    
    double dist_1_d = sqrt(pow((start_point->x - diam->r_point.x),2) + pow((start_point->y - diam->r_point.y),2));
    double dist_1_2 = sqrt(pow((start_point->x - other_point->x),2) + pow((start_point->y - other_point->y),2));
    if(dist_1_d != 0 && dist_1_2 != 0)  diameter_position_ratio = dist_1_d / dist_1_2;
    else diameter_position_ratio = .5;
}

// Given a point 'rpoint' (in image coordinates), returns the closest point on the
// The int* segment_start will be set to the starting point of the segment containing the diameter
// root instance 'r'.
// Note:  The function first converts to client coordinates to handle roots that span
// multiple windows, then it converts back to image coordinates before returning.
// used to be // RPoint CRootflyDlg::FindNearestPointOnRoot(const RPoint& rpoint, const RootInstance& r ) const
RPoint CRootflyDlg::FindNearestPointOnRootSeg(const RPoint& rpoint, const RootInstance& r, int* segment_start ) const
{
  RPoint out;
  double dist, min_dist = 99999.9;     
  const vector<RPoint> *line = r.GetAllPoints();
  assert( line->size() >= 2 );
  if (line->size() < 2)  return RPoint(-1, -1, -1);
  vector<RPoint>::const_iterator p1, p2;
  CPoint temp;
  
  p1 = &line->at(0);
  p2 = &line->at(1);
  *segment_start = 0;
  int count = 0;

  // for each line segment of the root instance, ...
  while ( p2 != line->end() )
  {
    // convert to client coordinates
    CPoint pp1 = m_converter.ImageToClient(*p1);
    CPoint pp2 = m_converter.ImageToClient(*p2);
    CPoint ppp = m_converter.ImageToClient(rpoint);

    // compute the distance to the segment, and store the minimum distance
    pt2d a,b,p,c; 
    a.x = pp1.x;
    a.y = pp1.y;
    b.x = pp2.x;
    b.y = pp2.y;
    p.x = ppp.x;
    p.y = ppp.y;
    dist = DistPt2Seg(a, b, p, &c);   
    if ( dist < min_dist )
    {
      min_dist = dist;
      temp.x = (int)(c.x);
      temp.y = (int)(c.y);
	  *segment_start = count;
    }
	count ++;
    p1++;
    p2++;
  } 

  // convert back to image coordinates
  return m_converter.ClientToImage(temp);
}

//Wrapper for FindNearestPointOnRootSeg for people not interested in the segment
RPoint CRootflyDlg::FindNearestPointOnRoot(const RPoint& rpoint, const RootInstance& r ) const
{
  int dummy;
  RPoint point = FindNearestPointOnRootSeg( rpoint, r, &dummy); //call with NULL pointer
  point.window = rpoint.window;
  return point;
}


//void CRootflyDlg::FindNearestPointOnRoot( const RPoint& rpoint, const RootInstance& r, RDiameter *temp ) const
//{  
//  int x = rpoint.x;
//  int y = rpoint.y;
//  pt2d a,b,p,c; 
//  double dist, min_dist = 99999.9;     
//  const vector<RPoint> *line = r.GetAllPoints();
//  vector<RPoint>::const_iterator p1;
//  vector<RPoint>::const_iterator p2;
//  if (line->size() < 2) return;
//  
//  p1 = &line->at(0);
//  p2 = &line->at(1);   
//  while ( p2 != line->end() )
//  {
//    a.x = p1->x;
//    a.y = p1->y;
//    b.x = p2->x;
//    b.y = p2->y;
//    p.x = x;
//    p.y = y;
//    dist = DistPt2Seg(a,b,p,&c);   
//    if ( dist < min_dist )
//    {
//      min_dist = dist;
//      temp->r_point.x = (int)(c.x);
//      temp->r_point.y = (int)(c.y);
//      temp->radius = min_dist;
//    } 
//    p1++;
//    p2++;	  
//  } 
//}
//
//
//CPoint CRootflyDlg::FindNearestPointOnRoot( const RPoint& rpoint, const RootInstance& r ) const
//{  
//  RDiameter dia;
//  FindNearestPointOnRoot( rpoint, r, &dia );
//  return CPoint( dia.r_point.x, dia.r_point.y );
//}

/*
CPoint CRootflyDlg::FindNearestPointOnRoot( const RPoint& rpoint, int ID ) const
{
  int x = rpoint.x;
  int y = rpoint.y;
  pt2d a,b,p,c; 
  CPoint temp;
  temp.r_point.x = 0;
  temp.r_point.y = 0;
  temp.r_point.window = -1;
  temp.radius = -1;
  
  bool first_time = true;
  double dist, min_dist;
  const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
  const std::vector<Root> *root = tube->GetAllRoots();
  std::vector<Root>::const_iterator root_ptr;
  const std::vector<RootInstance> *ri;
  std::vector<RootInstance>::const_iterator line;
  std::vector<RPoint>::const_iterator p1;
  std::vector<RPoint>::const_iterator p2;
  int window = m_app_data->GetWindow();
  
  for ( root_ptr = root->begin(); root_ptr != root->end(); root_ptr++ )
  {
    if ( root_ptr->GetID() != ID ) continue;
    ri = root_ptr->GetAllRootInstances();
    for ( line = ri->begin(); line != ri->end(); line++ )
    { 
      if ( line->GetSessionNumber() == m_app_data->GetSession() )
      {
        p1 = &(line->GetAllPoints()->at(0));
        p2 = &(line->GetAllPoints()->at(1));   
          while( p2 != line->GetAllPoints()->end())
          {     
            a.x = p1->x;
            a.y = p1->y;
            b.x = p2->x;
            b.y = p2->y;
            p.x = x;
            p.y = y;
            dist = DistPt2Seg(a,b,p,&c);   
            if ( first_time || dist < min_dist)
            {
              min_dist = dist;
              temp.r_point.x = (int)(c.x);
              temp.r_point.y = (int)(c.y);
              temp.radius = min_dist;
              first_time = false;
            } 
            p1++;
            p2++;
          }
        }
      }
    }
  }
  return temp;
}
*/

int CRootflyDlg::GetSelectedIndex( CPoint pt ) const
{
  if (m_selected_root_id == -1)  return NOTHING_SELECTED;
  CRect rect( pt.x-20, pt.y-20, pt.x+20, pt.y+20 );
  const RPoint *rpt;
  const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
  assert(tube);  if (!tube)  return NOTHING_SELECTED;
  const Root *root = tube->GetRoot(m_selected_root_id);
  assert(root);  if (!root)  return NOTHING_SELECTED;
  const RootInstance *inst = root->GetRootInstance(m_app_data->GetSession());
  assert(inst);  if (!inst)  return NOTHING_SELECTED;
  if ( m_set_data->ShowDiametersIsEnabled() )
  { 
    CRect diameter_top;
    diameter_top.left = inst->GetDiameter(0)->r_point.x - 7;
    diameter_top.right =  inst->GetDiameter(0)->r_point.x + 7;
    diameter_top.top =  inst->GetDiameter(0)->r_point.y - inst->GetDiameter(0)->radius - 7;
    diameter_top.bottom =  inst->GetDiameter(0)->r_point.y - inst->GetDiameter(0)->radius + 7;
    if ( diameter_top.PtInRect(pt))
      return RESIZING_DIAMETER_SELECTED;

    const int threshold = 4;
    if ( IsPtInsideDiameter(pt.x, pt.y, inst->GetDiameter(0), threshold) )
    {
      return DIAMETER_SELECTED;
    }
  }  
  int size = inst->GetAllPoints()->size();
  for ( int i = 0; i < size; i++ )
  {
    rpt = inst->GetPoint(i);
    if ( rect.PtInRect(CPoint(rpt->x,rpt->y)) )
      return i;
  }  
  return NOTHING_SELECTED;
}

double CRootflyDlg::DotProduct(pt2d a, pt2d b)
{ 
  return a.x * b.x + a.y * b.y; 
} 

/**

   @brief This method calculates the minimum distance from a
          point to a line.
   INPUT:
       'a' and 'b':  the endpoints of the line segment
       'p':  the point
   OUTPUT:
       'c':  the point on the line segment closest to 'p'
             (either the intersection of the perpendicular line through 'p' with the line 
             segment, or one of the endpoints of the line segment if the intersection is
             outside the line segment.)
   RETURN:
       Returns the square of the distance from 'p' to the line segment.
*/

double CRootflyDlg::DistPt2Seg(pt2d a, pt2d b, pt2d p, pt2d *c)
{                           
  double x,dd; 
  
  pt2d d,n,pma;   
  d.x=b.x-a.x; 
  d.y=b.y-a.y;   
  dd=DotProduct(d,d);   
  n.x=d.x/dd;       /* let N=(B-A)/( (B-A)^2 ) */ 
  n.y=d.y/dd;   
  pma.x=p.x-a.x; 
  pma.y=p.y-a.y;   
  x=DotProduct(pma,n);   /* let x=(P-A).N */ 
  
  if (x<0.0)          *c=a;    
  else if (x>1.0)     *c=b;    
  else 
  { 
    c->x=a.x+x*d.x;  /* C=A+x*(B-A) */ 
    c->y=a.y+x*d.y; 
  }   
  d.x=p.x-c->x; 
  d.y=p.y-c->y; 
  
  return DotProduct(d,d); 
} 

// returns whether point (x,y) is inside circle defined by 'diam'
// radius is expanded by 'tolerance' 
bool CRootflyDlg::IsPtInsideDiameter(int x, int y, const RDiameter* diam, int tolerance)
{
  double dx = diam->r_point.x - x;
  double dy = diam->r_point.y - y;
  return sqrt( dx * dx + dy * dy ) < diam->radius + tolerance;
}

// old:
//// return distance from point to outer circle of radius;
//// returns 0 if on the circle, negative if inside the circle, and positive if outside.
//double CRootflyDlg::DistPt2Diameter(int x1, int y1, const RDiameter* diam)
//{
//  double dx = diam->r_point.x - x1;
//  double dy = diam->r_point.y - y1;
//  double dist = sqrt( dx * dx + dy * dy ) - diam->radius;
////  if (dist < 0)  dist = 0;
//  return dist; 
//}

/**
/        Enables/disable menu/buttons depending on mode set.         
/        User: Thai Nguyen (TVN)
*/
void CRootflyDlg::EnableDisable()
{
  CMenu *menu = GetMenu();
  bool file_loaded     = (GetMode() != MODE_NOFILELOADED);
  bool root_selected   = (GetMode() == MODE_ROOTSELECTED);
  bool image_displayed = ((GetMode() != MODE_NOIMAGEDISPLAYED) && file_loaded);
  bool file_loaded_e   = file_loaded ? MF_ENABLED : MF_GRAYED;
  bool image_displayed_e =image_displayed ? MF_ENABLED : MF_GRAYED;
  bool root_selected_e = root_selected ? MF_ENABLED : MF_GRAYED;
  bool date_included   = m_image_file_format.DateIsIncluded();
  bool tube_created = (m_exp_data->GetTube(m_app_data->GetTube()) != NULL) ? true: false;
  
  
  // handle message commands
  // disable only when no file loaded
  menu->EnableMenuItem(ID_FILE_SAVEAS        , file_loaded_e);
  menu->EnableMenuItem(ID_FILE_SAVE          , file_loaded_e);
  menu->EnableMenuItem(ID_FILE_EXPORT        , file_loaded_e);
  menu->EnableMenuItem(ID_EDIT_CHANGE_IMG_DIR, file_loaded_e);
  menu->EnableMenuItem(ID_EDIT_OPTIONS       , file_loaded_e);
  menu->EnableMenuItem(ID_EDIT_CONVERSION    , file_loaded_e);
  menu->EnableMenuItem(ID_EDIT_SESSION_DATES , date_included);
  menu->EnableMenuItem(ID_EDIT_ZOOM_OUT      , file_loaded_e);
  menu->EnableMenuItem(ID_EDIT_UNDO , m_controller.isUndoVectorEmpty());
  menu->EnableMenuItem(ID_EDIT_REDO , m_controller.isRedoVectorEmpty());
  
  
  // handle spin edit boxes
  // disable only when no file loaded
  m_edit_window.EnableWindow  (file_loaded);
  m_edit_tube.EnableWindow    (file_loaded);
  m_edit_session.EnableWindow (file_loaded);
  m_spin_window.EnableWindow  (file_loaded);
  m_spin_tube.EnableWindow    (file_loaded);
  m_spin_session.EnableWindow (file_loaded);
  
  // set spin boxes to read-only only when no file loaded
  m_edit_window.SetReadOnly   (!file_loaded);
  m_edit_tube.SetReadOnly     (!file_loaded);
  m_edit_session.SetReadOnly  (!file_loaded); 
  
  // handle properties boxes
  // enable only when root is selected
  m_combo_color.EnableWindow  (root_selected);
  m_combo_state.EnableWindow  (root_selected);
  m_combo_custom.EnableWindow (root_selected); 
  m_edit_diameter.SetReadOnly (!(root_selected || GetMode() == MODE_RESIZINGDIAMETER));
 
  // handle buttons
  // disable all only when no file loaded
  // enable extend endpoint, delete root, add diameter
  //     only when root is selected  
  m_button_new_root.EnableWindow        (image_displayed);
  m_button_extend_endpoints.EnableWindow(root_selected);
  m_button_delete_root.EnableWindow     (root_selected);
  m_button_move_all_roots.EnableWindow  (image_displayed);
  m_button_copy.EnableWindow            (file_loaded);
  m_check_zoom_out.EnableWindow         (file_loaded);
  
  // menu items corresponding to buttons
  menu->EnableMenuItem(IDC_BUTTON_New_Root,         image_displayed_e);
  menu->EnableMenuItem(IDC_BUTTON_Extend_Endpoints, root_selected_e);
  menu->EnableMenuItem(IDC_BUTTON_Delete_Root,      root_selected_e);
  menu->EnableMenuItem(IDC_BUTTON_Move_All_Roots,   image_displayed_e);
  menu->EnableMenuItem(IDC_BUTTON_Copy_Session,     file_loaded_e);  
  menu->EnableMenuItem(IDC_CHECK_Zoom_Out,          file_loaded_e);
  
  // menu items (navigate)
  menu->EnableMenuItem(ID_KEYBOARD_UPARROW,         file_loaded_e);
  menu->EnableMenuItem(ID_KEYBOARD_DOWNARROW,       file_loaded_e);
  menu->EnableMenuItem(ID_KEYBOARD_LEFTARROW,       file_loaded_e);
  menu->EnableMenuItem(ID_KEYBOARD_RIGHTARROW,      file_loaded_e);  
  menu->EnableMenuItem(ID_NAVIGATE_PROGRESSWINDOW,  file_loaded_e); 

  //ANTHONY
  //menu->CheckMenuItem(ID_VIEW_RIGHTPANE,			TRUE);
  

  // check boxes
  m_checkbox_show_dead_roots.EnableWindow( file_loaded );
  m_checkbox_show_diameters.EnableWindow( file_loaded );
  m_checkbox_show_root_ids.EnableWindow( file_loaded );
} 

// returns true if in zoomed state, false otherwise
bool CRootflyDlg::IsZoomOutMode() const
{
  if (m_hWnd)
  {
    // old:  using Edit->Zoom
    unsigned isChecked = GetMenu()->GetSubMenu(1)->GetMenuState(ID_EDIT_ZOOM_OUT, MF_BYCOMMAND); 
    if (isChecked == MF_CHECKED)
       return true;
    else
       return (m_check_zoom_out.GetCheck() != 0);
 
    //   return ((ret & MF_CHECKED) != 0);
    // new:  using check box on main dialog    
  }
  else
  {
    return false;
  }
}

void CRootflyDlg::OnClose() 
{
  //check to see if any modifications have been made to the file
  int return_value;
  
  if (m_app_data->GetModify())
  {
    return_value = AfxMessageBox("Project data has changed. Save changes?", MB_YESNOCANCEL);
    if (return_value == IDCANCEL)  return;
    else if (return_value == IDYES)
    {
      OnFileSave();  //call the save function if things have changed
    }
  }
  
  // save settings to registry
  m_controller.C_SaveUserPreferences();

  CDialog::OnClose();
}

// brings up the dialog for mm conversion
void CRootflyDlg::OnConversion() 
{
  MMConversionDlg convert;
  CString str;
  convert.SetConversion(m_exp_data->GetMMConversion());
  convert.SetLineColor(m_set_data->GetRootDrawingColorSelected());
  
  int ret = convert.DoModal();
  if (ret == IDOK)
  {
    
    float conversion = convert.GetConversion();
    m_controller.C_SetMMConversion(conversion);
  }
}

// Move all roots (called 'Register' in version 1.0.3)
void CRootflyDlg::OnButtonMoveAllRoots() 
{
  //plan
  //change to mode picking start point
		//lbutton down, hold start point
		//change mode to moving all roots
  
  //on mouse move update all roots in window and redraw
  //on left click, change mode back
  
  SetMode(MODE_STARTINGMOVEALL);
  m_selected_root_id = -1;
  
  ReDraw(1,0);
}

void CRootflyDlg::OnCHECKZoomOut() 
{
  if (GetMenu()->GetSubMenu(1)->GetMenuState(ID_EDIT_ZOOM_OUT, MF_BYCOMMAND) == MF_CHECKED)
		GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_OUT, MF_UNCHECKED);
  else
    	GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_OUT, MF_CHECKED);

  CClientDC dc(this);
  m_graphics.FillImageDisplayAreaWithGray(dc,GetClientRectOfImageDisplayArea());
  SetZoomWindows();
  ReDraw(1, 0);	
}



void CRootflyDlg::OnEditChangeImgDir() 
{
  // Let user choose folder that contains images for the new project
  UINT	uFlags	  = BIF_RETURNONLYFSDIRS | BIF_USENEWUI;
  CString image_directory;
  
  bool valid_directory = false;
  while(!valid_directory)
  {
    CFolderDialog dlg(_T("Choose Folder where Images are Located" ), image_directory, this, uFlags);
    int fdlgid = dlg.DoModal();
    if (fdlgid == IDOK)
    {   
      image_directory = dlg.GetFolderPath();
      image_directory += "\\";
      if (m_image_file_format.ImageFileExistsInDirectory(image_directory))
        valid_directory = true;
      else
      {
        AfxMessageBox("No valid images were found in the directory");
      }
    }
    else return;
  }
  
  // We have valid directory. Let's save it.
  m_controller.C_SetImageDir(image_directory);
  m_controller.C_SetModify(true);
  
  // Set initial tube, window, and session info
  ImageInfo ii = m_image_file_format.GetImageInfoOfFirstFile(image_directory);
  m_controller.C_SetExperiment(ii.expt);
  m_controller.C_SetTube(ii.tube);
  m_controller.C_SetWindow(ii.window);
  m_converter.SetWindow( m_app_data->GetWindow() );
  m_controller.C_SetSession(ii.session);
  
  SetNavigation(image_directory);
  if (!m_image_file_format.DateIsIncluded())
  {
    SessionDatesDlg dlg;
    std::vector<Date> dates;
    dlg.SetSessionDates(m_exp_data->GetSessionDates());
    int ret = dlg.DoModal();
    if (ret == IDOK)
    {
      dates = dlg.GetSessionDates();
      m_controller.C_SetSessionDates(dates);      
      m_controller.C_SetModify(true);
    }
  }
  
  // get main image file name
  CString file_name = m_image_file_format.GetFileName(image_directory,m_app_data->GetTube(),m_app_data->GetWindow(),m_app_data->GetSession());
  
  SetMode(MODE_IMAGEDISPLAYED);
  
  m_controller.C_SetCurrentImage(file_name);
  
  ReDraw(1, 1);
}

void CRootflyDlg::OnEditColors() 
{
  CColorDlg colordlg(&m_controller, m_set_data, this);
  int id = colordlg.DoModal();
  if (id == IDOK)
  {
    ReDraw(1, 0);
  }
}


// Window will move up/down 1 by using the mouse wheel
BOOL CRootflyDlg::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
  if (GetMode() != MODE_NOFILELOADED)
  {
    if (zDelta > 0) ThumbnailClicked(-1,0);
    else ThumbnailClicked(1,0);
  }
  
  return CDialog::OnMouseWheel(nFlags, zDelta, pt);
}

// display the help menu when F1 is pressed 
BOOL CRootflyDlg::OnHelpInfo(HELPINFO* pHelpInfo) 
{
  OnHelpContents();
  return true;
}


//Will move up one window when up arrow is pressed
void CRootflyDlg::OnUpArrow()
{
  if (GetMode() != MODE_NOFILELOADED)
  {
    ThumbnailClicked(-1,0);
  }
}

//Will move down one window when down arrow is pressed
void CRootflyDlg::OnDownArrow()
{
  if (GetMode() != MODE_NOFILELOADED)
  {
    ThumbnailClicked(1,0);
  }
}

//Will move right one session when right arrow is pressed
void CRootflyDlg::OnRightArrow()
{
  if (GetMode() != MODE_NOFILELOADED)
  {
    ThumbnailClicked(1,1);
  }
}

//Will move left one session when left arrow is pressed
void CRootflyDlg::OnLeftArrow()
{
  if (GetMode() != MODE_NOFILELOADED)
  {
    ThumbnailClicked(-1,1);
  }
}

void CRootflyDlg::OnDeletePoint()
{ 
  m_controller.C_DeletePoint(m_selected_index, m_selected_root_id);
  const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
  if ( !tube ) {AfxMessageBox("No tube");return;}
  const Root *root = tube->GetRoot(m_selected_root_id);
  if ( !root ) {AfxMessageBox("No root");return;}
  const RootInstance *inst = root->GetRootInstance(m_app_data->GetSession());
  if ( !inst ) {AfxMessageBox("No inst");return;}
  const RDiameter *diam = inst->GetDiameter(0);
  if ( !diam ) {AfxMessageBox("No diam");return;}
  
  RPoint temp = FindNearestPointOnRoot( diam->r_point, *inst );
  if ( temp.x < 0 || temp.y < 0) return;
  double dist = sqrt(pow((temp.x - diam->r_point.x),2) + pow((temp.y - diam->r_point.y),2));
  if ( dist > 1.0 )  //move the diameter if it is not attached to the root anymore
  {
    m_controller.C_MoveDiameter(0, m_selected_root_id, temp);
  }
  
  SetMode(MODE_ROOTSELECTED);  
  ReDraw(1,0);
}

void CRootflyDlg::OnAddPoint()
{ 
  const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
  if ( !tube ) {AfxMessageBox("No tube");return;}
  const Root *root = tube->GetRoot(m_selected_root_id);
  if ( !root ) {AfxMessageBox("No root");return;}
  const RootInstance *inst = root->GetRootInstance(m_app_data->GetSession());
  if ( !inst ) {AfxMessageBox("No inst");return;}
  const RDiameter *diam = inst->GetDiameter(0);
  if ( !diam ) {AfxMessageBox("No diam");return;}

  RPoint *rpt = m_controller.C_CreateMidPoint(m_selected_root_id, section_start_point_index);
  rpt->x = temporary_point_to_add.x;
  rpt->y = temporary_point_to_add.y;
  rpt->window = diam->r_point.window;

  //Set Mode to move new point
  m_selected_index = section_start_point_index + 1;
  SetMode(MODE_MOVINGRPOINT); 
  UpdateDiameterPositionRatio();
  
  
  ReDraw(1,0);
}

void CRootflyDlg::OnDebugResetmode() 
{
  CString str;
  str.Format("Current mode: %d.  Please notify developer.", GetMode());
  AfxMessageBox(str);
  SetMode(MODE_IMAGEDISPLAYED);
  ClearSelection();
}

// This status text provides two purposes:  It informs the user about possible actions, and it provides
// information about the current 'mode' to aid debugging.  In order for the second purpose to be accomplished,
// be sure that the text is unique for each mode.
void CRootflyDlg::UpdateStatusText()
{
  CString str; 

  //STATUSBAR
  Status_bar.SetPaneText(0,"Started");

  switch( m_mode )
  {
  case      MODE_NOFILELOADED:			  // when app first starts up
    str += _T("To begin, choose File->New or File->Open");
    break;
  case      MODE_NOIMAGEDISPLAYED:		// no image for the current Tube/window/session
    str += _T("Navigate to a different tube, window, or session");
    break;
  case      MODE_IMAGEDISPLAYED:      // image displayed
    str += _T("Ready to annotate");
    break;  
  case      MODE_RESIZINGDIAMETER:    // resizing diameter
    str += _T("Resizing diameter, left click to finish");
    break;
  case      MODE_DRAWINGNEWROOT:      // drawing a new root (before first point clicked)
    str += _T("Left click to add point, double click to end root");
    break;
  case      MODE_EXTENDINGENDPOINT:   // extending endpoint of selected root
    str += _T("Left click to extend endpoint");
    break;
  case      MODE_LOCATINGDIAMETER:    // locating position of diameter for new root
    str += _T("Left click to place diameter");
    break;
  case      MODE_ROOTSELECTED:  
                                      // four cases may occur when the root is selected
    switch( m_selected_index )
    {
    case -3:
      str+= _T("Left click to resize this diameter");
      break;
    case -2:
      str+= _T("Left click to move this diameter");
      break;
    case -1:
      if(m_over_root) str += _T("Left click to move root or right click to add a new point");
      else str += _T("Ready to modify root");
      break;
    default:
      str += _T("Left click to move this point, right click to delete");
    }

    break;
  case      MODE_MOVINGRPOINT:        // moving point on selected root
    str += _T("Moving point, left click to finish");
    break;
  case      MODE_STARTINGMOVEALL:     // this state occurs before starting (x,y) are set
    str += _T("Left click to begin moving all roots");
    break;
  case      MODE_MOVINGSINGLEROOT:    // moving single root
    str += _T("Moving root, left click to finish");
    break;
  case      MODE_MOVINGALLROOTS:      // moving all roots
    str += _T("Moving roots, left click to finish");
    break;
  case      MODE_REPOSITIONDIAMETER:  // moving diameter of selected root
    str += _T("Moving diameter, left click to finish");
    break;
  case      MODE_SIZINGDIAMETER:      // changing size of diameter for new root
    str += _T("Left click to set diameter size");
    break;
  default:
    assert(0);
    str += _T("Program unstable.  Click Debug->Reset Mode");
    break;
  }
  

//  CString foo;
//  foo.Format(" (Code: %d)", m_mode);
//  str += foo;
  
  //m_status_text.SetWindowText(str);
  //STATUSBAR
  Status_bar.SetPaneText(0,str);
  CWnd* pApplyButton = GetDlgItem (IDC_STATIC_status);
  ASSERT (pApplyButton);
  pApplyButton->ShowWindow (SW_HIDE);
}

CDebugDlg* g_debug_dlg = NULL;
void CRootflyDlg::OnDebugDisplayrootdata() 
{
  // format data into a string
  CString str;
  str = "***** Roots in this window *****\r\n";
  {
    const Experiment* exp = m_rfy_data->GetExperiment();   
    if (exp == NULL) { AfxMessageBox("No experiment to display");  return; }
    const Tube *tube = exp->GetTube(m_app_data->GetTube());
    if (tube == NULL) { AfxMessageBox("No root data to display");  return; }
    const vector<Root> *root = tube->GetAllRoots();
    if (root == NULL) { AfxMessageBox("No roots in this tube");  return; }
    CRect main; 
    CPoint conv_point;
//    double rad; 
    CString tmpstr;
  
    int window = m_app_data->GetWindow();
    for ( int i = 0; i < root->size(); i++ )
    {
      const Root& rr = root->at(i);
      if ( rr.GetPrimaryWindow() == window)
      {
        tmpstr.Format("Root %3d (%3d) (%3d): [%3d,%3d] w=%3d\r\n", i, rr.GetID(), rr.GetWindowID(), rr.GetFirstSessionNumber(), rr.GetLastSessionNumber(), rr.GetPrimaryWindow());
        str += tmpstr;
        
        const std::vector<RootInstance>* rii = rr.GetAllRootInstances();
        for ( int j = 0 ; j < rii->size() ; j++)
        {
          const RootInstance& ri = rii->at(j);

          tmpstr.Format("  Session %3d:  %s ", ri.GetSessionNumber(), ri.IsAlive() ? "LIVE" : "DEAD");
          str += tmpstr;
          const std::vector<RPoint>* pp = ri.GetAllPoints();
          for ( int k = 0 ; k < pp->size() ; k++)
          {
            const RPoint& rp = pp->at(k);
            tmpstr.Format("(%3d,%3d,%3d) ", rp.x, rp.y, rp.window);
            str += tmpstr;
          }
          const RDiameter* rd = ri.GetDiameter(0);
          if (rd)  
          {
            tmpstr.Format(" dia=(%3d,%3d,%3d;%3.0f)", rd->r_point.x, rd->r_point.y, rd->r_point.window, rd->radius);
            str += tmpstr;
          }
          str += "\r\n";
        }
      }
    }
  }

  // display data
  if (g_debug_dlg == NULL)
  {
    g_debug_dlg = new CDebugDlg;
    g_debug_dlg->Create(CDebugDlg::IDD, this);
  }
  g_debug_dlg->SetText(str);
  g_debug_dlg->ShowWindow( SW_SHOW );
}

void CRootflyDlg::OnDebugRepairdiameter() 
{
  if (m_selected_root_id < 0)  return;

  // get selected root	
  CRect main;
  CPoint conv_point;
  vector<RPoint> extended_line;
  const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
  if (!tube)  return;
  const Root *root = tube->GetRoot( m_selected_root_id );
  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(m_app_data->GetSession());
  if (!inst)  return;
  const std::vector<RPoint> *line = inst->GetAllPoints();
  if (!line)  return;
  conv_point = m_converter.ImageToClient( CPoint( line->at(0).x, line->at(0).y), m_app_data->GetWindow() );
  const RDiameter *diam = inst->GetDiameter(0);
  if (!diam)  return;
  
  RDiameter* ddiam = const_cast<RDiameter*>( diam );
  ddiam->r_point.window = m_app_data->GetWindow();

  m_controller.C_SetModify(true);
  ReDraw(1, 0);
}

void CRootflyDlg::LeftClickWhileRootSelected( RPoint point )
{
  RDiameter nearest_point;
  FindNearestPoint( point.x, point.y, &nearest_point);      
  int selected_diameter = FindNearestDiameter(point.x, point.y);
  // Check to see if another root has been selected and change selected ID
  {
    bool clicked_point_of_selected_root = nearest_point.radius >= 0 && nearest_point.r_point.window == m_selected_root_id;
    bool clicked_diameter_of_selected_root = selected_diameter >= 0 && selected_diameter == m_selected_root_id;
    bool clicked_point_of_another_root = nearest_point.radius >= 0 && nearest_point.r_point.window != m_selected_root_id;
    bool clicked_diameter_of_another_root = selected_diameter >= 0 && selected_diameter != m_selected_root_id;
    if (!clicked_point_of_selected_root && !clicked_diameter_of_selected_root)
    {
      if (clicked_diameter_of_another_root) 
      {
        m_selected_root_id = selected_diameter;
        ReDraw(1,0);
        return;
      }
      else if (clicked_point_of_another_root) 
      {
        m_selected_root_id = nearest_point.r_point.window;  
        ReDraw(1,0);
        return;
      }
    }
  }
  // -1 means not over an interactive area of the root
  // -2 means over the diameter move area   (now moving the diameter)
  // -3 means over the diameter resize area (now resizing the diameter)   
  // all other indexes refer to points along the root (now moving a point)
  if (m_selected_index == RESIZING_DIAMETER_SELECTED) // -3
  {
    SetCursor(m_cur_sizens);
    SetMode(MODE_RESIZINGDIAMETER);
  }
  else if ( m_selected_index == DIAMETER_SELECTED )  // -2
  {
    SetCursor(m_cur_closedhand);
    SetMode(MODE_REPOSITIONDIAMETER);
  }
  else if ( m_selected_index >= 0 )
  {
    SetCursor(m_cur_sizeall);
    SetMode(MODE_MOVINGRPOINT); 
    UpdateDiameterPositionRatio();
  }
  else if (!m_over_root) 
  {
    // clicked off of root, it is no longer selected
    SetMode(MODE_IMAGEDISPLAYED);      
    m_selected_root_id = -1;
    m_selected_index = -1;       
    CPoint c_point; // = m_converter.ClientToImage( CPoint(point.x, point.y), &point.window);   
    c_point.x = point.x;
    c_point.y = point.y;
    if (!CheckForThumbnailClick(c_point))
    {
      ReDraw(1, 0);
    }
  }
  else
  {
    //preserve the starting point of the move 
    m_controller.C_SetShowRootID(false); //fix clipping error
    m_last_mouse_move_point = CPoint(point.x, point.y);
    SetMode(MODE_MOVINGSINGLEROOT);
  }
}

void CRootflyDlg::LeftClickWhileSizingDiameter( RPoint point )
{
  // done creating new root; create root in data structure and copy values over
  RDiameter *rd = m_current_root_being_drawn.GetDiameter(0);
  const Tube *tube = m_exp_data->GetTube( m_app_data->GetTube() );
  if (!tube) tube = m_controller.C_CreateTube(m_app_data->GetTube());
  const std::vector<RPoint> *line;  
  line = m_current_root_being_drawn.GetAllPoints();    
  if (line->size() == 0)  return;
  Root *root = m_controller.C_CreateRoot( line->at(0).window );    
  RootInstance *inst = m_controller.C_CreateRootInstance( root->GetID() );  
  RPoint *rpt;
  vector<RPoint>::const_iterator iter;
  for ( iter = line->begin(); iter != line->end(); iter++ )
  {
    rpt = inst->CreatePoint();
    rpt->x = iter->x;
    rpt->y = iter->y;
    rpt->window = iter->window;     
  }
  RDiameter *rd2 = m_controller.C_CreateDiameter( root->GetID() );
  rd2->r_point.x = rd->r_point.x;
  rd2->r_point.y = rd->r_point.y;
  rd2->r_point.window = rd->r_point.window;
  rd2->radius = rd->radius;
  // sets the default condition code for the root
  SetConditionDefault(root);
  m_selected_root_id = -1;
  SetMode(MODE_IMAGEDISPLAYED);
  m_current_root_being_drawn.ClearAll();
  ReDraw(1, 0);
}

void CRootflyDlg::LeftClickWhileExtendingEndpoint( RPoint point )
{
  const std::vector<RPoint> *line;
  line = m_current_root_being_drawn.GetAllPoints();
  RDiameter *rd = m_current_root_being_drawn.GetDiameter(0);
  const Tube *tube = m_exp_data->GetTube( m_app_data->GetTube() );
  assert(tube); if (!tube) return;
  const vector<Root> *roots = tube->GetAllRoots();
  assert(roots); if (!roots) return;;    
  for ( int i = 0; i < roots->size(); i++ )
  {
    if ( roots->at(i).GetID() == m_selected_root_id )
    {
      const RootInstance *inst = roots->at(i).GetRootInstance(m_app_data->GetSession());  
      if (!inst) return;
      const std::vector<RPoint> *old_line = inst->GetAllPoints();
      if (m_extending_from_tail)
      {      
        if (point.x == line->back().x && point.y == line->back().y) 
        {
          m_extending_from_tail = false;
          SetMode(MODE_ROOTSELECTED);
          ReDraw(1,0);
          return;
        }
        RPoint *rpt = m_controller.C_CreatePoint(roots->at(i).GetID());
        rpt->x = point.x;
        rpt->y = point.y;
        rpt->window = point.window;

//		    // Stupid fixing stupid.
//		    
//		    if(m_converter.GetWindow()>point.window)
//			    rpt->window = point.window+1;
//		    else if(m_converter.GetWindow()<point.window)
//			    rpt->window = point.window-1;
//		    else 
//			    rpt->window = point.window;


        m_current_root_being_drawn.ClearAll();
        m_extending_from_tail = false;
        SetMode(MODE_ROOTSELECTED);
        ReDraw(1,0);
        return;
      }
      if (m_extending_from_head)
      {        
        for ( int j = 0; j < old_line->size(); j++ )
        {
          RPoint temp_point;
          temp_point.x = old_line->at(j).x;
          temp_point.y = old_line->at(j).y;
          temp_point.window = old_line->at(j).window;
          m_controller.C_MovePoint(j,roots->at(i).GetID(), point.x, point.y);
          m_controller.C_ChangeWindowOfPoint(j,roots->at(i).GetID(), point.window);
          point.x = temp_point.x;
          point.y = temp_point.y;
          point.window = temp_point.window;
        }
        RPoint *displaced_rpt = m_controller.C_CreatePoint(roots->at(i).GetID());
        displaced_rpt->x = point.x;
        displaced_rpt->y = point.y;
        displaced_rpt->window = point.window;

//        // stupid fix
//		    if(m_converter.GetWindow()<point.window)
//			    displaced_rpt->window = point.window-1;
//		    else if(m_converter.GetWindow()>point.window)
//			    displaced_rpt->window = point.window+1;
//		    else  
//			    displaced_rpt->window = point.window;
        m_current_root_being_drawn.ClearAll();
        m_extending_from_head = false;
        SetMode(MODE_ROOTSELECTED);
        ReDraw(1,0);        
      }
    }
  } 
}

void CRootflyDlg::LeftClickWhileDrawingNewRoot( RPoint point )
{
  SetCursor(m_cur_pen);
  RPoint *rpt = m_current_root_being_drawn.CreatePoint();    
  assert(rpt); if( !rpt ) return;
  rpt->x = point.x;
  rpt->y = point.y;

  rpt->window = point.window;

//  // stupid fix
//  if(m_converter.GetWindow()<point.window)
//		rpt->window = point.window-1;
//	else if(m_converter.GetWindow()>point.window)
//		rpt->window = point.window+1;
//	else  
//		rpt->window = point.window;        
  ReDraw(1, 0);    
}

void CRootflyDlg::LeftClickWhileMovingSingleRoot( RPoint point )
{
  //finished moving root
  SetMode(MODE_IMAGEDISPLAYED);
  if (m_checkbox_show_root_ids.GetCheck() == BST_CHECKED)
  {
    m_controller.C_SetShowRootID(true);
  }
  ReDraw(1,0);
}

void CRootflyDlg::LeftClickWhileMovingAllRoots( RPoint point )
{
  //finished with move all
  SetMode(MODE_IMAGEDISPLAYED);
  if (m_checkbox_show_root_ids.GetCheck() == BST_CHECKED)
  {
    m_controller.C_SetShowRootID(true);
  }
  ReDraw(1,0);    
}

void CRootflyDlg::LeftClickWhileStartingMoveAll( RPoint point )
{
  //preserve the starting point of the move all
  m_controller.C_SetShowRootID(false); //fix clipping error
  m_last_mouse_move_point = CPoint(point.x, point.y);
  SetMode(MODE_MOVINGALLROOTS);
}

void CRootflyDlg::MoveMouseWhileMovingRPoint( RPoint point )
{
  SetCursor(m_cur_sizeall);
  RPoint tempdiam;
  const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
  assert(tube);  if (!tube)  return;
  const Root *root = tube->GetRoot(m_selected_root_id);
  assert(root);  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(m_app_data->GetSession());
  assert(inst);  if (!inst)  return;
  const RDiameter *diam = inst->GetDiameter(0);
  assert(diam);  if (!diam)  return;
  const RPoint oldpoint = *( inst->GetPoint(m_selected_index) );

  m_controller.C_MovePoint( m_selected_index, m_selected_root_id, point.x, point.y );
  m_controller.C_ChangeWindowOfPoint( m_selected_index, m_selected_root_id, point.window );

  // check if diameter is on relevant root segment, if it isn't, do nothing.
  if ( (section_start_point_index == m_selected_index) || (section_start_point_index == (m_selected_index-1) ) )
  {
    //get other point of segment
    const RPoint* otherpoint; 
    //get new diameter position if neccesary
    if ( section_start_point_index == m_selected_index )
    {
      otherpoint = inst->GetPoint(section_start_point_index + 1);
      tempdiam.x = (double)point.x - ( (double)point.x - (double)otherpoint->x ) * diameter_position_ratio;
      tempdiam.y = (double)point.y - ( (double)point.y - (double)otherpoint->y ) * diameter_position_ratio;
    }
    else
    {
      otherpoint = inst->GetPoint(section_start_point_index);
      tempdiam.x = (double)point.x - ( (double)point.x - (double)otherpoint->x ) * (1 - diameter_position_ratio);
      tempdiam.y = (double)point.y - ( (double)point.y - (double)otherpoint->y ) * (1 - diameter_position_ratio);
    }
    tempdiam.window = diam->r_point.window;
    m_controller.C_MoveDiameter(0, m_selected_root_id, tempdiam);
  }
  ReDraw(1, 0);
}

void CRootflyDlg::MoveMouseWhileMovingRDiameter( RPoint point )
{
  SetCursor(m_cur_closedhand);
  const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
  assert(tube);  if (!tube)  return;
  const Root *root = tube->GetRoot(m_selected_root_id);
  assert(root);  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(m_app_data->GetSession());
  assert(inst);  if (!inst)  return;
  const RDiameter *diam = inst->GetDiameter(0);
  assert(diam);  if (!diam)  return;
  RPoint temp = FindNearestPointOnRoot(point, *inst);
  if ( temp.x < 0 || temp.y < 0) return;
  double dist = sqrt(pow((temp.x - point.x),2) + pow((temp.y - point.y),2));       
  if ( dist > 1.0 )  //keeps the diameter from walking around
  {
    m_controller.C_MoveDiameter(0, m_selected_root_id, temp);   
  } 
  ReDraw(1, 0);
}

void CRootflyDlg::MoveMouseWhileResizingRDiameter( RPoint point )
{
  SetCursor(m_cur_sizens);    
  const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
  assert(tube);  if (!tube)  return;
  const Root *root = tube->GetRoot(m_selected_root_id);
  assert(root);  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(m_app_data->GetSession());
  assert(inst);  if (!inst)  return;
  const RDiameter *diam = inst->GetDiameter(0);
  assert(diam);  if (!diam)  return;
  {        
    int delta_x, delta_y;        
    delta_x = diam->r_point.x - point.x;
    delta_y = diam->r_point.y - point.y;
    double new_radius = sqrt(pow(delta_x,2)+pow(delta_y,2));
    m_controller.C_ResizeDiameter(0, m_selected_root_id, new_radius);      
  }
  ReDraw(1,0);
}

void CRootflyDlg::MoveMouseWhileRootSelected( RPoint point )
{
  RDiameter nearest_point;
  nearest_point.radius = -1;
  FindNearestPoint( point.x, point.y, &nearest_point);   
  if ( nearest_point.radius < 0 )        
  {
    m_over_root = false;      
    UpdateStatusText();
  }
  else
  {
    m_over_root = true;   
    UpdateStatusText();
  }
  m_selected_index = GetSelectedIndex(CPoint( point.x, point.y ));    
  if ( m_selected_index == RESIZING_DIAMETER_SELECTED )
  {
    SetCursor(m_cur_sizens);
    UpdateStatusText();
  }
  else if ( m_selected_index == DIAMETER_SELECTED )
  {
    SetCursor(m_cur_openhand);
    UpdateStatusText();
  }
  else if ( m_selected_index == NOTHING_SELECTED )
  {
  }
  else if ( m_selected_index >= 0 )
  {
    SetCursor(m_cur_sizeall); 
    UpdateStatusText();
  }  
}

void CRootflyDlg::MoveMouseWhileImageDisplayed( RPoint point )
{
  int tempRootID;	
  RDiameter nearest_point;
  FindNearestPoint( point.x, point.y, &nearest_point);      
  int diameter_selected = FindNearestDiameter( point.x, point.y );
  if ( nearest_point.radius < 0 && diameter_selected < 0)
  {
    m_selected_root_id = -1;
    if ( m_over_root )
    {
      m_over_root = false; 
      ReDraw(1, 0);          
    }
  }
  
  else 
  { 
	const Tube *tube = m_exp_data->GetTube(m_app_data->GetTube());
	if (!tube)  return;
    if ( nearest_point.radius >= 0)
	{
	  tempRootID = nearest_point.r_point.window;
	  const Root *root = tube->GetRoot(tempRootID);
	  if (!root)  return;
	  const RootInstance *inst = root->GetRootInstance(m_app_data->GetSession());
	  if (!inst)  return;
	  if ((inst->IsAlive()) || (m_checkbox_show_dead_roots.GetCheck() == BST_CHECKED))
	    m_selected_root_id = nearest_point.r_point.window; 
	}
    else
	{
	  tempRootID = diameter_selected;
	  const Root *root = tube->GetRoot(tempRootID);
	  if (!root)  return;
	  const RootInstance *inst2 = root->GetRootInstance(m_app_data->GetSession());
	  if (!inst2)  return;
      if ((inst2->IsAlive()) || (m_checkbox_show_dead_roots.GetCheck() == BST_CHECKED))
	    m_selected_root_id =  diameter_selected;   
	}	
    if ( !m_over_root )
    {
	  m_over_root = true;
      ReDraw(1, 0);
    }
    else      ReDraw(1, 0);    
  }
  
}


void CRootflyDlg::MoveMouseWhileExtendingEndpoint( RPoint point )
{
  CPoint head = m_converter.ImageToClient(*m_current_root_being_drawn.GetPoint(0));
  const vector<RPoint> *points = m_current_root_being_drawn.GetAllPoints();    
  CPoint tail = m_converter.ImageToClient(*m_current_root_being_drawn.GetPoint(points->size()-1));
  CPoint pt = m_converter.ImageToClient(point);

  double delta_x, delta_y;
  delta_x = pt.x - head.x;
  delta_y = pt.y - head.y;
  double distance_to_head = delta_x * delta_x + delta_y * delta_y;
  delta_x = pt.x - tail.x;
  delta_y = pt.y - tail.y;
  double distance_to_tail = delta_x * delta_x + delta_y * delta_y;
  m_extending_from_tail = (distance_to_tail < distance_to_head) ? true : false;
  m_extending_from_head = (distance_to_tail < distance_to_head) ? false : true;
  ReDraw(1,0);

//  int delta_x, delta_y;
//  delta_x = point.x - m_current_root_being_drawn.GetPoint(0)->x;
//  delta_y = point.y - m_current_root_being_drawn.GetPoint(0)->y;
//  float distance_to_head = sqrt(pow(delta_x,2) + pow(delta_y,2));
//  const vector<RPoint> *points = m_current_root_being_drawn.GetAllPoints();    
//  delta_x = point.x - m_current_root_being_drawn.GetPoint(points->size()-1)->x;
//  delta_y = point.y - m_current_root_being_drawn.GetPoint(points->size()-1)->y;
//  float distance_to_tail = sqrt(pow(delta_x,2) + pow(delta_y,2));
//  m_extending_from_tail = (distance_to_tail < distance_to_head)?true:false;
//  m_extending_from_head = (distance_to_tail < distance_to_head)?false:true;
//  ReDraw(1,0);
}

void CRootflyDlg::MoveMouseWhileMovingSingleRoot( RPoint point )
{
  CString temp;
  CPoint last = m_last_mouse_move_point;
  //get session number
  m_edit_session.GetWindowText(temp);
  int session = atoi(temp);		
  m_controller.C_MoveRootSess( m_selected_root_id, session, point.x - last.x, point.y - last.y );   
  //update the new old point
  m_last_mouse_move_point = CPoint( point.x, point.y );
  ReDraw(1,0);
}

void CRootflyDlg::MoveMouseWhileMovingAllRoots( RPoint point )
{
  int window = m_app_data->GetWindow();
  CPoint last = m_last_mouse_move_point;
  if (point.y == 507)
	m_controller.C_MoveRootsInWindow(window, point.x - last.x, abs(point.y - last.y));
  else
	m_controller.C_MoveRootsInWindow(window, point.x - last.x, point.y - last.y);    
  //update the new old point
  m_last_mouse_move_point = CPoint( point.x, point.y );
  ReDraw(1,0);
}

void CRootflyDlg::MoveMouseWhileSizingDiameter( RPoint point )
{
  RDiameter *rdm = m_current_root_being_drawn.GetDiameter(0);
  rdm->radius = sqrt( pow(( point.x - rdm->r_point.x),2) + pow(( point.y - rdm->r_point.y),2) ); 
  ReDraw(1, 0);  
}

void CRootflyDlg::MoveMouseWhileLocatingDiameter( RPoint point )
{
  RDiameter *rdm = m_current_root_being_drawn.GetDiameter(0);
  RPoint temp = FindNearestPointOnRoot( point, m_current_root_being_drawn);    
  rdm->r_point.x = temp.x;
  rdm->r_point.y = temp.y;
  rdm->r_point.window = temp.window; 
  rdm->radius = 8;    
  ReDraw(1, 0); 
}


void CRootflyDlg::OnNavigateProgresswindow() 
{
	FindAnnotatedRoots();
  
  ProgressDlg dlg(&m_progress_data, this);
  dlg.DoModal();
}

void CRootflyDlg::OnEditAutodetect() 
{
  CAutoDetectDlg autodlg(&m_controller, m_set_data, this);
  autodlg.DoModal();
}

//this is helper method for the ProgressWindow Class
//it will find if a file exists and if it does it will set it as existing in the class
void CRootflyDlg::FindAnnotatedRoots()
{
	ImageFileFormat ii = m_controller.C_GetImageFileFormat();
	int window_count = m_exp_data->GetMaxWindows();
	int session_count = m_exp_data->GetMaxSessions();
	int tube_count = m_exp_data->GetNTubes();
	CString fileName;
	//ProgressData test;
	
  int t, w, s;

  m_progress_data.Reset();
	for(t=1;t<=tube_count;t++){
		for(w=1;w<=window_count;w++){
			for(s=1;s<=session_count;s++){
				CString tube("00"+IntToString(t));
				fileName = ii.GetFileName(m_exp_data->GetImageDir(),tube,w,s);
				if(fileName.GetLength()!=0){
					m_progress_data.SetExists(t,w,s);
				}
			}
		}
	}

  if (m_mode == MODE_NOFILELOADED)
    return;

  const std::vector<Tube> *tubes;
  const std::vector<Root> *roots;
  const Tube *tube;
  const Root *root;

  tubes = m_exp_data->GetAllTubes();
  for (tube = tubes->begin(); tube < tubes->end(); tube++)
  {
    roots = tube->GetAllRoots();
    for (root = roots->begin(); root < roots->end(); root++)
    {
       w = root->GetPrimaryWindow();
       s = root->GetFirstSessionNumber();

       if ((w > window_count) || (s > session_count))
         continue;

       m_progress_data.SetAnnotated(atoi(tube->GetTag()), w, s);
    }
  }
}

CString CRootflyDlg::IntToString(int i)
{
	std::stringstream ss;
	ss<<i;
	return ss.str().c_str();
}

void CRootflyDlg::OnEditZoomOut() 
{
	// TODO: Add your command handler code here
	
	if(GetMenu()->GetSubMenu(1)->GetMenuState(ID_EDIT_ZOOM_OUT, MF_BYCOMMAND) == MF_CHECKED)
	{
		GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_OUT, MF_UNCHECKED);
		m_check_zoom_out.SetCheck(BST_UNCHECKED);
	}// if
	else
	{
    	GetMenu()->GetSubMenu(1)->CheckMenuItem(ID_EDIT_ZOOM_OUT, MF_CHECKED);
		m_check_zoom_out.SetCheck(BST_CHECKED);
	}// else

    CClientDC dc(this);
    m_graphics.FillImageDisplayAreaWithGray(dc,GetClientRectOfImageDisplayArea());
    SetZoomWindows();
    ReDraw(1, 0);	
}

void CRootflyDlg::OnEditUndo() 
{
	m_controller.Undo();
	ReDraw(1, 0);
	EnableDisable();
}

void CRootflyDlg::OnEditRedo() 
{
	m_controller.Redo();
	ReDraw(1, 0);	
	EnableDisable();
}

//This function toggles the check mark next to the Right Pane Option on the View Menu
//then calls the function to redraw the window
void CRootflyDlg::OnUpdateViewRightpane(CCmdUI* pCmdUI) 
{
	CMenu* mmenu = GetMenu();
	UINT state = mmenu->GetMenuState(ID_VIEW_RIGHTPANE, MF_UNCHECKED);

	if(state == MF_UNCHECKED){
		mmenu->CheckMenuItem(ID_VIEW_RIGHTPANE,MF_CHECKED);

	}
	else{
		mmenu->CheckMenuItem(ID_VIEW_RIGHTPANE,MF_UNCHECKED);
	}

	RECT m_windowSize;
	GetClientRect(&m_windowSize);
	OnSizeNew(7,m_windowSize.right, m_windowSize.bottom);

}

//when the window is redrawn, this fucntion will be called if the 
// Right Pane option of the view menu is uncheck
// then it will hide all of the objects contained in the right pane
void CRootflyDlg::Rootfly_Right_Objects_Hide()
{
  //AfxMessageBox("HIDE OBJECTS");
  	//CMenu* mmenu = GetMenu();
	  //UINT state = mmenu->GetMenuState(ID_VIEW_RIGHTPANE, MF_UNCHECKED);

  	CWnd* pApplyButton = GetDlgItem (IDC_STATIC_Properties_Section);
    ASSERT (pApplyButton);


		//mmenu->CheckMenuItem(ID_VIEW_RIGHTPANE,MF_UNCHECKED);
		//AfxMessageBox("overview status now unchecked:HIDE");
        pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_STATIC_Tube);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_STATIC_Tube);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_EDIT_TUBE);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_SPIN_TUBE);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_STATIC_Window);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_EDIT_WINDOW);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_SPIN_WINDOW);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_STATIC_Session);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_EDIT_SESSION);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

///
		pApplyButton = GetDlgItem (IDC_SPIN_SESSION);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_STATIC_Root_ID);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_EDIT_ROOTID);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_STATIC_Diameter);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_EDIT_DIAMETER);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_STATIC_Length);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_EDIT_LENGTH);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_STATIC_Color);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);
///

		pApplyButton = GetDlgItem (IDC_COMBO_COLOR);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_STATIC_State);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_COMBO_STATE);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_STATIC_Custom);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_COMBO_CUSTOM);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_CHECK_Show_Root_IDs);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_CHECK_Show_Dead_Roots);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_CHECK_Show_Diameters);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_BUTTON_New_Root);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);
////

		pApplyButton = GetDlgItem (IDC_BUTTON_Extend_Endpoints);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_BUTTON_Delete_Root);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_BUTTON_Move_All_Roots);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_BUTTON_Copy_Session);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);

		pApplyButton = GetDlgItem (IDC_CHECK_Zoom_Out);
		ASSERT (pApplyButton);
		pApplyButton->ShowWindow (SW_HIDE);


}


void CRootflyDlg::OnBUTTONAutoDetect() 
{
  RunAutomaticRootDetector();	
  ReDraw(1,0);	
}

//This function toggles the check mark next to the Left Pane Option on the View Menu
//then calls the function to redraw the window
void CRootflyDlg::OnUpdateViewLeftpane(CCmdUI* pCmdUI) 
{
	if(GetMenu()->GetMenuState(ID_VIEW_LEFTPANE, MF_BYCOMMAND) == MF_CHECKED){
		GetMenu()->CheckMenuItem(ID_VIEW_LEFTPANE, MF_UNCHECKED);
	}
	else{
		GetMenu()->CheckMenuItem(ID_VIEW_LEFTPANE, MF_CHECKED);
	}

	RECT m_windowSize;
	GetClientRect(&m_windowSize);
	OnSizeNew(8,m_windowSize.right, m_windowSize.bottom);
	
}

//This function toggles the check mark next to the Bottom Pane Option on the View Menu
//then calls the function to redraw the window
void CRootflyDlg::OnUpdateViewBottompane(CCmdUI* pCmdUI) 
{
	if(GetMenu()->GetMenuState(ID_VIEW_BOTTOMPANE, MF_BYCOMMAND) == MF_CHECKED){
		GetMenu()->CheckMenuItem(ID_VIEW_BOTTOMPANE, MF_UNCHECKED);
	}
	else{
		GetMenu()->CheckMenuItem(ID_VIEW_BOTTOMPANE, MF_CHECKED);
	}

	RECT m_windowSize;
	GetClientRect(&m_windowSize);
	OnSizeNew(9,m_windowSize.right, m_windowSize.bottom);
}


//when the window is redrawn, this fucntion will be called if the 
// Left Pane option of the view menu is uncheck
// then it will hide all of the objects contained in the left pane
void CRootflyDlg::Rootfly_Left_Objects_Hide(){
	GetDlgItem(IDC_SCROLLBAR2)->ShowWindow (SW_HIDE);
	GetDlgItem(IDC_WIND_PIC_1)->ShowWindow (SW_HIDE);
	GetDlgItem(IDC_WIND_PIC_2)->ShowWindow (SW_HIDE);
	GetDlgItem(IDC_WIND_PIC_3)->ShowWindow (SW_HIDE);
	GetDlgItem(IDC_WIND_PIC_4)->ShowWindow (SW_HIDE);
	GetDlgItem(IDC_WIND_PIC_5)->ShowWindow (SW_HIDE);
	GetDlgItem(IDC_BUTTON_Auto_Detect)->ShowWindow (SW_HIDE);
	GetDlgItem(IDC_STATIC_Window_Section)->ShowWindow (SW_HIDE);
}


//when the window is redrawn, this fucntion will be called if the 
// Bottom Pane option of the view menu is uncheck
// then it will hide all of the objects contained in the bottom pane
void CRootflyDlg::Rootfly_Bottom_Objects_Hide(){
	GetDlgItem(IDC_SCROLLBAR3)->ShowWindow (SW_HIDE);
	GetDlgItem(IDC_SESS_PIC_1)->ShowWindow (SW_HIDE);
	GetDlgItem(IDC_SESS_PIC_2)->ShowWindow (SW_HIDE);
	GetDlgItem(IDC_SESS_PIC_3)->ShowWindow (SW_HIDE);
	GetDlgItem(IDC_SESS_PIC_4)->ShowWindow (SW_HIDE);
	GetDlgItem(IDC_SESS_PIC_5)->ShowWindow (SW_HIDE);
	GetDlgItem(IDC_STATIC_Session_Section)->ShowWindow (SW_HIDE);
}
